@@ -219,7 +219,7 @@ goc_graph_update_bounds (GocItem *item)
 	item->y0 = graph->y;
 	item->x1 = graph->x + graph->w;
 	item->y1 = graph->y + graph->h;
-	gog_renderer_update (graph->renderer, 
+	gog_renderer_update (graph->renderer,
 				      (int) (graph->w * item->canvas->pixels_per_unit),
 				      (int) (graph->h * item->canvas->pixels_per_unit));
 }
@@ -235,8 +235,7 @@ format_coordinate (GogAxis *axis, GOFormat *fmt, double x)
 		 go_format_measure_strlen,
 		 go_font_metrics_unit,
 		 fmt,
-		 x, 'F', NULL,
-		 NULL,
+		 x, 'F', NULL, NULL,
 		 width, date_conv, TRUE);
 	if (err) {
 		/* Invalid number for format.  */
@@ -246,19 +245,21 @@ format_coordinate (GogAxis *axis, GOFormat *fmt, double x)
 	return g_string_free (res, FALSE);
 }
 
+#ifdef GOFFICE_WITH_GTK
 static void
 goc_graph_do_tooltip (GocGraph *graph)
 {
-	GogView *view;
+	GogView *view, *base_view;
 	char *buf = NULL, *s1 = NULL, *s2 = NULL;
 	GogObject *obj;
 	GogChart *chart;
 	GogViewAllocation alloc;
 	double xpos, ypos;
-	GogChartMap *map;
-	GogAxis *x_axis, *y_axis;
+	GogChartMap *map = NULL;
+	GogAxis *x_axis, *y_axis = NULL;
 	GogAxisSet set;
-	GSList *l;
+	GogPlot *plot;
+	GSList *l, *ptr;
 	GOFormat *format;
 	GocItem *item = (GocItem *)graph;
 	double x = graph->coords.x;
@@ -270,11 +271,15 @@ goc_graph_do_tooltip (GocGraph *graph)
 	goc_group_adjust_coords (item->parent, &xpos, &ypos);
 	x -= xpos;
 	y -= ypos;
+	/* multiply by the zoom level, because the graph has been adjusted,
+	 * fixes #657694 */
+	x *= item->canvas->pixels_per_unit;
+	y *= item->canvas->pixels_per_unit;
 
 	/* get the GogView at the cursor position */
-	g_object_get (G_OBJECT (graph->renderer), "view", &view, NULL);
-	g_object_unref (view); /* we don't need a reference */
-	gog_view_get_view_at_point (view, x, y, &obj, NULL);
+	g_object_get (G_OBJECT (graph->renderer), "view", &base_view, NULL);
+	g_object_unref (base_view); /* we don't need a reference */
+	gog_view_get_view_at_point (base_view, x, y, &obj, NULL);
 	if (!obj)
 		goto tooltip;
 	chart = GOG_CHART (gog_object_get_parent_typed (obj, GOG_TYPE_CHART));
@@ -285,35 +290,57 @@ goc_graph_do_tooltip (GocGraph *graph)
 	l = gog_object_get_children (GOG_OBJECT (chart), gog_object_find_role_by_name (GOG_OBJECT (chart), "Plot"));
 	if (l == NULL)
 		return;
-	view = gog_view_find_child_view (view, GOG_OBJECT (l->data));
+	ptr = l;
+	while (ptr) {
+		view = gog_view_find_child_view (base_view, GOG_OBJECT (ptr->data));
+		if (view) {
+			double start, end;
+			alloc = view->allocation;
+			plot = GOG_PLOT (ptr->data);
+			switch (set) {
+			case GOG_AXIS_SET_XY:
+				/* get the axis */
+				x_axis = gog_plot_get_axis (plot, GOG_AXIS_X);
+				gog_axis_get_effective_span (x_axis, &start, &end);
+				if (x < alloc.x + alloc.w * start || x > alloc.x + alloc.w * end)
+					break;
+				y_axis = gog_plot_get_axis (plot, GOG_AXIS_Y);
+				gog_axis_get_effective_span (y_axis, &start, &end);
+				if (y < alloc.y + alloc.h * (1. - start) && y > alloc.y + alloc.h * (1. - end))
+					map = gog_chart_map_new (chart, &alloc, x_axis, y_axis, NULL, FALSE);
+				break;
+			case GOG_AXIS_SET_RADAR: {
+				double a, r, min, max;
+				/* get the axis */
+				x_axis = gog_plot_get_axis (plot, GOG_AXIS_CIRCULAR);
+				y_axis = gog_plot_get_axis (plot, GOG_AXIS_RADIAL);
+				map = gog_chart_map_new (chart, &alloc, x_axis, y_axis, NULL, FALSE);
+				gog_axis_get_bounds (y_axis, &min, &max);
+				r = min - 1;
+				if (gog_chart_map_is_valid (map))
+					gog_chart_map_view_to_2D (map, x, y, &a, &r);
+				if (r < min || r > max) {
+					gog_chart_map_free (map);
+					map = NULL;
+				}
+				break;
+			}
+			default:
+				buf = gog_view_get_tip_at_point (view, x, y);
+				g_slist_free (l);
+				goto tooltip;
+			}
+			if (map != NULL)
+				break;
+			view = NULL;
+		}
+		ptr = ptr->next;
+	}
 	g_slist_free (l);
-	if (!view)
+	if (view == NULL) {
+		gtk_widget_set_tooltip_markup (GTK_WIDGET (item->canvas), NULL);
 		return;
-	alloc = view->allocation;
-	switch (set) {
-	case GOG_AXIS_SET_XY:
-		/* get the axis */
-		l = gog_chart_get_axes (chart, GOG_AXIS_X);
-		x_axis = GOG_AXIS (l->data);
-		g_slist_free (l);
-		l = gog_chart_get_axes (chart, GOG_AXIS_Y);
-		y_axis = GOG_AXIS (l->data);
-		g_slist_free (l);
-		break;
-	case GOG_AXIS_SET_RADAR:
-		/* get the axis */
-		l = gog_chart_get_axes (chart, GOG_AXIS_CIRCULAR);
-		x_axis = GOG_AXIS (l->data);
-		g_slist_free (l);
-		l = gog_chart_get_axes (chart, GOG_AXIS_RADIAL);
-		y_axis = GOG_AXIS (l->data);
-		g_slist_free (l);
-		break;
-	default:
-		buf = gog_view_get_tip_at_point (view, x, y);
-		goto tooltip;
 	}
-	map = gog_chart_map_new (chart, &alloc, x_axis, y_axis, NULL, FALSE);
 	if (gog_chart_map_is_valid (map) &&
 				x >= alloc.x && x < alloc.x + alloc.w &&
 				y >= alloc.y && y < alloc.y + alloc.h) {
@@ -377,7 +404,7 @@ goc_graph_motion (GocItem *item, double x, double y)
 				       graph);
 	}
 
-	/* When the timer first, use the last (x,y) we have.  */
+	/* When the timer fires, use the last (x,y) we have.  */
 	graph->coords.x = x;
 	graph->coords.y = y;
 
@@ -396,6 +423,7 @@ goc_graph_leave_notify (GocItem *item, double x, double y)
 	gtk_widget_set_tooltip_text (GTK_WIDGET (item->canvas), NULL);
 	return ((GocItemClass*) parent_klass)->leave_notify (item, x, y);
 }
+#endif
 
 static void
 goc_graph_class_init (GocItemClass *item_klass)
@@ -448,8 +476,10 @@ goc_graph_class_init (GocItemClass *item_klass)
 	item_klass->draw = goc_graph_draw;
 	item_klass->update_bounds = goc_graph_update_bounds;
 	item_klass->distance = goc_graph_distance;
+#ifdef GOFFICE_WITH_GTK
 	item_klass->leave_notify = goc_graph_leave_notify;
 	item_klass->motion = goc_graph_motion;
+#endif
 }
 
 GSF_CLASS (GocGraph, goc_graph,
