@@ -60,11 +60,11 @@ goc_path_set_property (GObject *gobject, guint param_id,
 	case PATH_PROP_ROTATION:
 		path->rotation = g_value_get_double (value);
 		break;
-		
+
 	case PATH_PROP_CLOSED:
 		path->closed = g_value_get_boolean (value);
 		break;
-		
+
 	case PATH_PROP_PATH:
 		path->path = g_value_peek_pointer (value);
 		break;
@@ -191,8 +191,8 @@ goc_path_draw (GocItem const *item, cairo_t *cr)
 
 	cairo_save(cr);
 	if (goc_path_prepare_draw (item, cr, 1)) {
-		if (path->closed && go_styled_object_set_cairo_fill (GO_STYLED_OBJECT (item), cr))
-			cairo_fill_preserve (cr);
+		if (path->closed)
+			go_styled_object_fill (GO_STYLED_OBJECT (item), cr, TRUE);
 		if (goc_styled_item_set_cairo_line (GOC_STYLED_ITEM (item), cr)) {
 			cairo_stroke (cr);
 		} else {
