@@ -22,28 +22,6 @@
 #include <goffice/goffice-config.h>
 #include <goffice/goffice.h>
 #include <goffice/goffice-priv.h>
-#include <goffice/graph/gog-series.h>
-#include <goffice/graph/gog-plot.h>
-#include <goffice/graph/gog-plot-engine.h>
-#include <goffice/graph/gog-chart.h>
-#include <goffice/graph/gog-graph.h>
-#include <goffice/graph/gog-axis.h>
-#include <goffice/graph/gog-legend.h>
-#include <goffice/graph/gog-label.h>
-#include <goffice/graph/gog-grid.h>
-#include <goffice/graph/gog-grid-line.h>
-#include <goffice/graph/gog-theme.h>
-#include <goffice/graph/gog-error-bar.h>
-#include <goffice/graph/gog-series-lines.h>
-#include <goffice/graph/gog-3d-box.h>
-#include <goffice/data/go-data-simple.h>
-#include <goffice/math/go-distribution.h>
-#include <goffice/math/go-math.h>
-#include <goffice/utils/go-format.h>
-#include <goffice/utils/go-font.h>
-#include <goffice/app/go-conf.h>
-#include <goffice/app/go-plugin-service.h>
-#include <goffice/component/go-component-factory.h>
 #include <gsf/gsf-utils.h>
 
 #include "goffice-paths.h"
@@ -57,6 +35,10 @@
 #undef STRICT
 #endif
 
+#ifdef GTKOSXAPPLICATION
+#include <gtkosxapplication.h>
+#endif
+
 #ifdef GOFFICE_WITH_LASEM
 #include <goffice/graph/gog-equation.h>
 #endif
@@ -67,6 +49,7 @@ static char const *libgoffice_data_dir   = GOFFICE_DATADIR;
 static char const *libgoffice_icon_dir   = GOFFICE_ICONDIR;
 static char const *libgoffice_locale_dir = GOFFICE_LOCALEDIR;
 static char const *libgoffice_lib_dir    = GOFFICE_LIBDIR;
+static char const *libgoffice_extern_plugin_dir = GOFFICE_EXTERNPLUGINDIR;
 
 /**
  * SECTION: goffice
@@ -125,6 +108,12 @@ go_sys_lib_dir (void)
 	return libgoffice_lib_dir;
 }
 
+gchar const *
+go_sys_extern_plugin_dir (void)
+{
+	return libgoffice_extern_plugin_dir;
+}
+
 
 /**
  * libgoffice_init:
@@ -165,6 +154,24 @@ libgoffice_init (void)
 		"lib", "goffice", GOFFICE_VERSION, NULL);
 	g_free (dir);
 	}
+#else
+#ifdef GTKOSXAPPLICATION
+    if (quartz_application_get_bundle_id ())
+    {
+        gchar *dir;
+
+        dir = quartz_application_get_resource_path ();
+	libgoffice_data_dir = g_build_filename (dir,
+		"share", "goffice", GOFFICE_VERSION, NULL);
+	libgoffice_icon_dir = g_build_filename (dir,
+		"share", "pixmaps", "goffice", GOFFICE_VERSION, NULL);
+	libgoffice_locale_dir = g_build_filename (dir,
+		"share", "locale", NULL);
+	libgoffice_lib_dir = g_build_filename (dir,
+		"lib", "goffice", GOFFICE_VERSION, NULL);
+	g_free (dir);
+    }
+#endif /* GTKOSXAPPLICATION */
 #endif
 
 	bindtextdomain (GETTEXT_PACKAGE, libgoffice_locale_dir);
@@ -176,6 +183,7 @@ libgoffice_init (void)
 	_go_conf_init ();
 	_go_fonts_init ();
 	_go_math_init ();
+	_go_rsm_init ();
 
 	/* keep trigger happy linkers from leaving things out */
 	_go_plugin_services_init ();
@@ -199,10 +207,16 @@ libgoffice_init (void)
 #endif
 	(void) GOG_TYPE_ERROR_BAR;
 	(void) GOG_TYPE_REG_EQN;
+	(void) GOG_TYPE_SERIES_LABELS;
+	(void) GOG_TYPE_DATA_LABEL;
 	(void) GOG_TYPE_SERIES_LINES;
 	(void) GO_TYPE_DATA_SCALAR_VAL;
 	(void) GO_TYPE_DATA_SCALAR_STR;
 	(void) GOG_3D_BOX_TYPE;
+	(void) GO_TYPE_EMF;
+	(void) GO_TYPE_PIXBUF;
+	(void) GO_TYPE_SPECTRE;
+	(void) GO_TYPE_SVG;
 	_gog_themes_init ();
 	_go_number_format_init ();
 	_go_currency_date_format_init ();
@@ -222,6 +236,7 @@ libgoffice_shutdown (void)
 	if (--initialized)
 		return;
 	_gog_themes_shutdown ();
+	_go_glib_extras_shutdown ();
 	_go_fonts_shutdown ();
 	_go_conf_shutdown ();
 #ifdef GOFFICE_WITH_GTK
@@ -232,6 +247,7 @@ libgoffice_shutdown (void)
 	_go_number_format_shutdown ();
 	_go_string_shutdown ();
 	_go_locale_shutdown ();
+	_go_rsm_shutdown ();
 #ifdef G_OS_WIN32
 	/* const_cast, we created these above */
 	g_free ((char *)libgoffice_data_dir);
