@@ -56,7 +56,9 @@ typedef struct _GogEquation	GogEquation;
 typedef struct _GogRegCurve	GogRegCurve;
 typedef struct _GogRegEqn	GogRegEqn;
 typedef struct _GogTrendLineType	GogTrendLineType;
-typedef struct _GogSeriesLines GogSeriesLines;
+typedef struct _GogSeriesLines  GogSeriesLines;
+typedef struct _GogSeriesLabels GogSeriesLabels;
+typedef struct _GogDataLabel    GogDataLabel;
 typedef struct _GogSmoothedCurve	GogSmoothedCurve;
 typedef struct _Gog3DBox	Gog3DBox;
 
@@ -87,7 +89,7 @@ typedef struct {
 } GogViewPadding;
 
 typedef void (*GogEnumFunc) (unsigned i, gpointer style,
-			     char const *name, gpointer data);
+			     char const *name, PangoAttrList *l, gpointer data);
 
 typedef enum {
 	GOG_AXIS_UNKNOWN = -1,
@@ -190,7 +192,13 @@ typedef enum {
 
 	GOG_POSITION_ANY_MANUAL      = 0xff80,
 
-	GOG_POSITION_PADDING	     = 1 << 16
+	GOG_POSITION_PADDING	     = 1 << 16,
+
+	GOG_POSITION_MANUAL_W        = 1 << 21, /* relative width */
+	GOG_POSITION_MANUAL_W_ABS    = 1 << 22, /* absolute width */
+	GOG_POSITION_MANUAL_H        = 1 << 23, /* relative height */
+	GOG_POSITION_MANUAL_H_ABS    = 1 << 24, /* absolute height */
+	GOG_POSITION_ANY_MANUAL_SIZE = 0xf00000
 } GogObjectPosition;
 
 typedef enum {
@@ -199,6 +207,18 @@ typedef enum {
 	GOG_PLOT_RENDERING_BEFORE_GRID
 } GogPlotRenderingOrder;
 
+typedef enum {
+	GOG_SERIES_LABELS_DEFAULT_POS = 0,
+	GOG_SERIES_LABELS_CENTERED = 1,
+	GOG_SERIES_LABELS_TOP = 1 << 1,
+	GOG_SERIES_LABELS_BOTTOM = 1 << 2,
+	GOG_SERIES_LABELS_LEFT = 1 << 3,
+	GOG_SERIES_LABELS_RIGHT = 1 << 4,
+	GOG_SERIES_LABELS_OUTSIDE = 1 << 5,
+	GOG_SERIES_LABELS_INSIDE = 1 << 6,
+	GOG_SERIES_LABELS_NEAR_ORIGIN = 1 << 7,
+} GogSeriesLabelsPos;
+
 #define GOG_POSITION_IS_SPECIAL(pos) (((pos) & GOG_POSITION_SPECIAL)&&(!((pos) & GOG_POSITION_MANUAL)))
 #define GOG_POSITION_IS_PADDING(pos) (((pos) & GOG_POSITION_PADDING)&&(!((pos) & GOG_POSITION_MANUAL)))
 
@@ -253,6 +273,7 @@ G_END_DECLS
 #include <goffice/graph/gog-plot.h>
 #include <goffice/graph/gog-plot-impl.h>
 #include <goffice/graph/gog-reg-curve.h>
+#include <goffice/graph/gog-series-labels.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-series-lines.h>
 #include <goffice/graph/gog-smoothed-curve.h>
