@@ -92,6 +92,7 @@ struct _GogAxis {
 
 	GogAxisTick	*ticks;
 	unsigned	 tick_nbr;
+	double span_start, span_end;    /* percent of used area */
 };
 
 /*****************************************************************************/
@@ -105,6 +106,29 @@ struct _GogAxis {
 
 static void gog_axis_set_ticks (GogAxis *axis,int tick_nbr, GogAxisTick *ticks);
 
+static PangoLayout *
+gog_get_layout (void)
+{
+	PangoContext *context = pango_context_new ();
+	PangoLayout *layout = pango_layout_new (context);
+	g_object_unref (context);
+	return layout;
+}
+
+static void
+gog_axis_ticks_set_text (GogAxisTick *ticks, char const *str)
+{
+	go_string_unref (ticks->str);
+	ticks->str = go_string_new (str);
+}
+
+static void
+gog_axis_ticks_set_markup (GogAxisTick *ticks, char const *str, PangoAttrList *l)
+{
+	go_string_unref (ticks->str);
+	ticks->str = go_string_new_rich (str, -1, l, NULL);
+}
+
 static GogAxisTick *
 create_invalid_axis_ticks (double min, double max)
 {
@@ -114,8 +138,9 @@ create_invalid_axis_ticks (double min, double max)
 	ticks[0].position = min;
 	ticks[1].position = max;
 	ticks[0].type = ticks[1].type = GOG_AXIS_TICK_MAJOR;
-	ticks[0].label = g_strdup ("##");
-	ticks[1].label = g_strdup ("##");
+	ticks[0].str = ticks[1].str = NULL;
+	gog_axis_ticks_set_text (&ticks[0], "##");
+	gog_axis_ticks_set_text (&ticks[1], "##");
 
 	return ticks;
 }
@@ -139,26 +164,37 @@ gog_axis_get_effective_format (GogAxis const *axis)
 	return axis->format;
 }
 
-static char *
-axis_format_value (GogAxis *axis, double val)
+static void
+axis_format_value (GogAxis *axis, double val, GOString **str)
 {
 	GOFormat *fmt = gog_axis_get_effective_format (axis);
-	GString *res = g_string_sized_new (20);
 	const GODateConventions *date_conv = axis->date_conv;
-	GOFormatNumberError err = go_format_value_gstring
-		(NULL, res,
+	GOFormatNumberError err;
+	PangoLayout *layout = gog_get_layout ();
+
+	g_return_if_fail (layout != NULL);
+
+	go_string_unref (*str);
+
+	err = go_format_value_gstring
+		(layout, NULL,
 		 go_format_measure_strlen,
 		 go_font_metrics_unit,
 		 fmt,
-		 val, 'F', NULL,
-		 NULL,
+		 val, 'F', NULL, NULL,
 		 -1, date_conv, TRUE);
-	if (err) {
-		/* Invalid number for format.  */
-		g_string_assign (res, "#####");
+	if (err)
+		*str = go_string_new ("#####");
+	else {
+		*str = go_string_new_rich
+			(pango_layout_get_text (layout), -1,
+			 pango_attr_list_ref
+			 (pango_layout_get_attributes (layout)),
+			 NULL);
+		*str = go_string_trim (*str, TRUE);
 	}
 
-	return g_string_free (res, FALSE);
+	g_object_unref (layout);
 }
 
 static gboolean
@@ -345,22 +381,36 @@ map_discrete_calc_ticks (GogAxis *axis)
 	for (i = 0; i < tick_nbr; i++) {
 		ticks[i].position = tick_start + (double) (i) * major_tick;
 		ticks[i].type = GOG_AXIS_TICK_MAJOR;
-		ticks[i].label = NULL;
+		ticks[i].str = NULL;
 	}
 	for (i = 0, j = tick_nbr; i < label_nbr; i++, j++) {
 		ticks[j].position = go_rint (label_start + (double) (i) * major_label);
 		index = ticks[j].position - 1;
 		ticks[j].type = GOG_AXIS_TICK_NONE;
+		ticks[j].str = NULL;
 		if (axis->labels != NULL) {
 			if (index < (int) go_data_get_vector_size (axis->labels) && index >= 0) {
-				char *label = go_data_get_vector_string (axis->labels, index);
-				ticks[j].label = g_markup_escape_text (label, -1);
-				g_free (label);
-			} else
-				ticks[j].label = NULL;
+				PangoAttrList *l = go_data_get_vector_markup (axis->labels, index);
+				if (l != NULL) {
+					char *label = go_data_get_vector_string (axis->labels, index);
+					gog_axis_ticks_set_markup (&ticks[j], label, l);
+					g_free (label);
+				} else {
+					double val = go_data_get_vector_value (axis->labels, index);
+					if (go_finite (val))
+						axis_format_value (axis, val, &ticks[j].str);
+					else {
+						char *label = go_data_get_vector_string (axis->labels, index);
+						gog_axis_ticks_set_text (&ticks[j], label);
+						g_free (label);
+					}
+				}
+			}
+		} else {
+			char *label = g_strdup_printf ("%d", index + 1);
+			gog_axis_ticks_set_text (&ticks[j], label);
+			g_free (label);
 		}
-		else
-			ticks[j].label = g_strdup_printf ("%d", index + 1);
 	}
 
 	gog_axis_set_ticks (axis, tick_nbr + label_nbr, ticks);
@@ -713,7 +763,10 @@ map_linear_calc_ticks (GogAxis *axis)
 	min_step = gog_axis_get_entry (axis, GOG_AXIS_ELEM_MINOR_TICK, NULL);
 	if (min_step <= 0.) min_step = maj_step;
 	while (1) {
-		double ratio = go_fake_floor (maj_step / min_step);
+		/* add 0.9 there because the ratio might not be an integer
+		 * or there might be rounding errors, more than 0.9 would
+		 * put a minor tick quite near the next major tick, see #657670) */
+		double ratio = go_fake_floor (maj_step / min_step + 0.9);
 		double Nd = (maj_N + 2) * ratio;
 		if (Nd >= 10 * GOG_AXIS_MAX_TICK_NBR)
 			min_step *= 10;
@@ -761,7 +814,7 @@ map_linear_calc_ticks (GogAxis *axis)
 			g_assert (t < N);
 			ticks[t].position = maj_pos;
 			ticks[t].type = GOG_AXIS_TICK_MAJOR;
-			ticks[t].label = axis_format_value (axis, maj_pos);
+			axis_format_value (axis, maj_pos, &ticks[t].str);
 			t++;
 		}
 
@@ -775,7 +828,7 @@ map_linear_calc_ticks (GogAxis *axis)
 			g_assert (t < N);
 			ticks[t].position = min_pos;
 			ticks[t].type = GOG_AXIS_TICK_MINOR;
-			ticks[t].label = NULL;
+			ticks[t].str = NULL;
 			t++;
 		}
 	}
@@ -985,7 +1038,7 @@ map_date_calc_ticks (GogAxis *axis)
 
 		ticks[t].position = maj_pos;
 		ticks[t].type = GOG_AXIS_TICK_MAJOR;
-		ticks[t].label = axis_format_value (axis, maj_pos);
+		axis_format_value (axis, maj_pos, &ticks[t].str);
 		t++;
 
 		/* Calculate next major so we know when to stop minors.  */
@@ -1007,7 +1060,7 @@ map_date_calc_ticks (GogAxis *axis)
 
 			ticks[t].position = min_pos;
 			ticks[t].type = GOG_AXIS_TICK_MINOR;
-			ticks[t].label = NULL;
+			ticks[t].str = NULL;
 			t++;
 		}
 	}
@@ -1035,6 +1088,7 @@ map_date_get_dim_format (GogAxis *axis, unsigned dim)
 {
 	switch (dim) {
 	case GOG_AXIS_ELEM_MIN:
+
 	case GOG_AXIS_ELEM_MAX: {
 		GOFormat *fmt = gog_axis_get_effective_format (axis);
 		/*
@@ -1349,7 +1403,7 @@ map_log_calc_ticks (GogAxis *axis)
 			g_assert (t < N);
 			ticks[t].position = maj_pos;
 			ticks[t].type = GOG_AXIS_TICK_MAJOR;
-			ticks[t].label = axis_format_value (axis, maj_pos);
+		        axis_format_value (axis, maj_pos, &ticks[t].str);
 			t++;
 		}
 
@@ -1373,7 +1427,7 @@ map_log_calc_ticks (GogAxis *axis)
 			g_assert (t < N);
 			ticks[t].position = min_pos;
 			ticks[t].type = GOG_AXIS_TICK_MINOR;
-			ticks[t].label = NULL;
+			ticks[t].str = NULL;
 			t++;
 		}
 	}
@@ -1439,7 +1493,7 @@ gog_axis_map_set_by_num (GogAxis *axis, gint num)
 }
 
 static void
-gog_axis_map_populate_combo (GogAxis *axis, GtkComboBox *combo)
+gog_axis_map_populate_combo (GogAxis *axis, GtkComboBoxText *combo)
 {
 	unsigned i;
 
@@ -1447,9 +1501,9 @@ gog_axis_map_populate_combo (GogAxis *axis, GtkComboBox *combo)
 
 	for (i = 0; i < G_N_ELEMENTS (map_descs); i++) {
 		const char *thisname = map_descs[i]->name;
-		gtk_combo_box_append_text (combo, _(thisname));
+		gtk_combo_box_text_append_text (combo, _(thisname));
 		if (!g_ascii_strcasecmp (thisname, axis->map_desc->name))
-			gtk_combo_box_set_active (combo, i);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i);
 	}
 }
 #endif
@@ -1536,6 +1590,11 @@ gog_axis_map_new (GogAxis *axis, double offset, double length)
 	map->data = NULL;
 	map->is_valid = FALSE;
 
+	if (axis->type != GOG_AXIS_CIRCULAR) {
+		offset += axis->span_start * length;
+		length *= axis->span_end - axis->span_start;
+	}
+
 	if (map->desc->init != NULL)
 		map->is_valid = map->desc->init (map, offset, length);
 
@@ -1671,6 +1730,11 @@ gog_axis_map_get_extents (GogAxisMap *map, double *start, double *stop)
 		map->desc->map_bounds (map, stop, start);
 	else
 		map->desc->map_bounds (map, start, stop);
+	if (map->axis->type != GOG_AXIS_CIRCULAR) {
+		double buf = (*stop - *start) / (map->axis->span_end - map->axis->span_start);
+		*start -= map->axis->span_start * buf;
+		*stop = *start + buf;
+	}
 }
 
 /**
@@ -1691,6 +1755,11 @@ gog_axis_map_get_bounds (GogAxisMap *map, double *minimum, double *maximum)
 	g_return_if_fail (map != NULL);
 
 	map->desc->map_bounds (map, minimum, maximum);
+	if (map->axis->type != GOG_AXIS_CIRCULAR) {
+		double buf = (*minimum - *maximum) / (map->axis->span_end - map->axis->span_start);
+		*maximum -= map->axis->span_start * buf;
+		*minimum = *maximum + buf;
+	}
 }
 
 /**
@@ -1778,7 +1847,7 @@ gog_axis_set_ticks (GogAxis *axis, int tick_nbr, GogAxisTick *ticks)
 
 	if (axis->ticks != NULL) {
 		for (i = 0; i < axis->tick_nbr; i++)
-			g_free (axis->ticks[i].label);
+			go_string_unref (axis->ticks[i].str);
 
 		g_free (axis->ticks);
 	}
@@ -1831,7 +1900,9 @@ enum {
 	AXIS_PROP_MAP,
 	AXIS_PROP_ASSIGNED_FORMAT_STR_XL,
 	AXIS_PROP_CIRCULAR_ROTATION,
-	AXIS_PROP_POLAR_UNIT
+	AXIS_PROP_POLAR_UNIT,
+	AXIS_PROP_SPAN_START,
+	AXIS_PROP_SPAN_END
 };
 
 /*****************************************************************************/
@@ -2024,6 +2095,20 @@ gog_axis_set_property (GObject *obj, guint param_id,
 		}
 		break;
 	}
+	case AXIS_PROP_SPAN_START:
+		if (axis->type != GOG_AXIS_CIRCULAR) {
+			double new_value = g_value_get_double (value);
+			g_return_if_fail (new_value < axis->span_end);
+			axis->span_start = new_value;
+		}
+		break;
+	case AXIS_PROP_SPAN_END:
+		if (axis->type != GOG_AXIS_CIRCULAR) {
+			double new_value = g_value_get_double (value);
+			g_return_if_fail (new_value > axis->span_start);
+			axis->span_end = new_value;
+		}
+		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
@@ -2067,6 +2152,12 @@ gog_axis_get_property (GObject *obj, guint param_id,
 	case AXIS_PROP_POLAR_UNIT:
 		g_value_set_string (value, polar_units[axis->polar_unit].name);
 		break;
+	case AXIS_PROP_SPAN_START:
+		g_value_set_double (value, axis->span_start);
+		break;
+	case AXIS_PROP_SPAN_END:
+		g_value_set_double (value, axis->span_end);
+		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -2304,7 +2395,7 @@ cb_update_dim_editor (GogObject *gobj, ElemToggleData *closure)
 }
 
 static void
-make_dim_editor (GogDataset *set, GtkTable *table, unsigned dim,
+make_dim_editor (GogDataset *set, GtkGrid *grid, unsigned dim,
 		 GogDataAllocator *dalloc, char const *dim_name)
 {
 	ElemToggleData *info;
@@ -2342,10 +2433,11 @@ make_dim_editor (GogDataset *set, GtkTable *table, unsigned dim,
 
 	g_object_weak_ref (G_OBJECT (toggle), (GWeakNotify) elem_toggle_data_free, info);
 
-	gtk_table_attach (table, toggle,
-		0, 1, dim + 1, dim + 2, GTK_FILL, 0, 0, 0);
-	gtk_table_attach (table, editor,
-		1, 2, dim + 1, dim + 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+	gtk_grid_attach (grid, toggle,
+		0, dim + 1, 1, 1);
+	g_object_set (G_OBJECT (editor), "hexpand", TRUE, NULL);
+	gtk_grid_attach (grid, editor,
+		1, dim + 1, 1, 1);
 
 }
 
@@ -2365,18 +2457,18 @@ cb_map_combo_changed (GtkComboBox *combo,
 }
 
 static void
-gog_axis_populate_polar_unit_combo (GogAxis *axis, GtkComboBox *combo)
+gog_axis_populate_polar_unit_combo (GogAxis *axis, GtkComboBoxText *combo)
 {
 	unsigned i, id = 0;
 
 	g_return_if_fail (GOG_IS_AXIS (axis));
 
 	for (i = 0; i < G_N_ELEMENTS (polar_units); i++) {
-		gtk_combo_box_append_text (combo, _(polar_units[i].name));
+		gtk_combo_box_text_append_text (combo, _(polar_units[i].name));
 		if (polar_units[i].unit == axis->polar_unit)
 			id = i;
 	}
-	gtk_combo_box_set_active (combo, id);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), id);
 }
 
 static void
@@ -2403,6 +2495,24 @@ cb_rotation_changed (GtkSpinButton *spin, GogAxis *axis)
 	gog_object_emit_changed (GOG_OBJECT (axis), TRUE);
 }
 
+static void
+cb_start_changed (GtkSpinButton *spin, GogAxis *axis)
+{
+	double val = gtk_spin_button_get_value (spin);
+	GtkAdjustment *adj = GTK_ADJUSTMENT (g_object_get_data (G_OBJECT (spin), "other-adj"));
+	gtk_adjustment_set_lower (adj, fmin (val + 1, axis->span_end * 100.));
+	g_object_set (axis, "span-start", val /100., NULL);
+}
+
+static void
+cb_end_changed (GtkSpinButton *spin, GogAxis *axis)
+{
+	double val = gtk_spin_button_get_value (spin);
+	GtkAdjustment *adj = GTK_ADJUSTMENT (g_object_get_data (G_OBJECT (spin), "other-adj"));
+	gtk_adjustment_set_upper (adj, fmax (val - 1, axis->span_start * 100.));
+	g_object_set (axis, "span-end", val /100., NULL);
+}
+
 static void
 gog_axis_populate_editor (GogObject *gobj,
 			  GOEditor *editor,
@@ -2414,14 +2524,14 @@ gog_axis_populate_editor (GogObject *gobj,
 		"invert-axis"
 	};
 	GtkWidget *w;
-	GtkTable  *table;
+	GtkGrid *grid;
 	unsigned i = 0;
 	GogAxis *axis = GOG_AXIS (gobj);
 	GogAxisPrefState *state;
 	GogDataset *set = GOG_DATASET (gobj);
 	GtkBuilder *gui;
 
-	gui = go_gtk_builder_new ("gog-axis-prefs.ui", GETTEXT_PACKAGE, cc);
+	gui = go_gtk_builder_new_internal ("gog-axis-prefs.ui", GETTEXT_PACKAGE, cc);
 	if (gui == NULL)
 		return;
 
@@ -2430,7 +2540,7 @@ gog_axis_populate_editor (GogObject *gobj,
 	g_object_ref (G_OBJECT (axis));
 
 	/* Bounds Page */
-	table = GTK_TABLE (gtk_builder_get_object (gui, "bound_table"));
+	grid = GTK_GRID (gtk_builder_get_object (gui, "bound-grid"));
 	if (axis->is_discrete) {
 		static char const * const dim_names[] = {
 			N_("M_inimum"),
@@ -2439,7 +2549,7 @@ gog_axis_populate_editor (GogObject *gobj,
 			N_("Categories between _labels")
 		};
 		for (i = GOG_AXIS_ELEM_MIN; i < GOG_AXIS_ELEM_CROSS_POINT ; i++)
-			make_dim_editor (set, table, i, dalloc,
+			make_dim_editor (set, grid, i, dalloc,
 					 _(dim_names[i]));
 	} else {
 		static char const * const dim_names[] = {
@@ -2450,40 +2560,42 @@ gog_axis_populate_editor (GogObject *gobj,
 		};
 
 		for (i = GOG_AXIS_ELEM_MIN; i < GOG_AXIS_ELEM_CROSS_POINT ; i++)
-			make_dim_editor (set, table, i, dalloc,
+			make_dim_editor (set, grid, i, dalloc,
 					 _(dim_names[i]));
 	}
-	gtk_widget_show_all (GTK_WIDGET (table));
+	gtk_widget_show_all (GTK_WIDGET (grid));
 
 	/* Details page */
 	if (!axis->is_discrete && gog_axis_get_atype (axis) != GOG_AXIS_CIRCULAR) {
-		GtkComboBox *box = go_gtk_builder_combo_box_init_text (gui, "map_type_combo");
+		GtkComboBoxText *box = GTK_COMBO_BOX_TEXT (gtk_builder_get_object (gui, "map-type-combo"));
 		gog_axis_map_populate_combo (axis, box);
 		g_signal_connect_object (G_OBJECT (box),
 					 "changed",
 					 G_CALLBACK (cb_map_combo_changed),
 					 axis, 0);
 	} else {
-		GtkWidget *w = go_gtk_builder_get_widget (gui, "map_type_box");
+		GtkWidget *w = go_gtk_builder_get_widget (gui, "map-label");
+		gtk_widget_hide (w);
+		w = go_gtk_builder_get_widget (gui, "map-type-combo");
 		gtk_widget_hide (w);
 	}
 
 	if (!axis->is_discrete && gog_axis_get_atype (axis) == GOG_AXIS_CIRCULAR) {
 		GtkWidget *w;
-		GtkComboBox *box = go_gtk_builder_combo_box_init_text (gui, "polar_unit_combo");
+		GtkComboBoxText *box = GTK_COMBO_BOX_TEXT (gtk_builder_get_object (gui, "polar-unit-combo"));
 		gog_axis_populate_polar_unit_combo (axis, box);
 		g_signal_connect (G_OBJECT (box),
 				  "changed",
 				  G_CALLBACK (cb_polar_unit_changed),
 				  state);
 
-		w = go_gtk_builder_get_widget (gui, "circular_rotation_spinbutton");
+		w = go_gtk_builder_get_widget (gui, "circular-rotation-spinbutton");
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), axis->circular_rotation);
 		g_signal_connect_object (G_OBJECT (w), "value-changed",
 					 G_CALLBACK (cb_rotation_changed),
 					 axis, 0);
 	} else {
-		GtkWidget *w = go_gtk_builder_get_widget (gui, "circular_table");
+		GtkWidget *w = go_gtk_builder_get_widget (gui, "circular-grid");
 		gtk_widget_hide (w);
 	}
 
@@ -2499,15 +2611,37 @@ gog_axis_populate_editor (GogObject *gobj,
 	}
 
 	go_editor_add_page (editor,
-			     go_gtk_builder_get_widget (gui, "axis_pref_box"),
+			     go_gtk_builder_get_widget (gui, "axis-pref-grid"),
 			     _("Scale"));
 
+	/* effective area */
+	if (axis->type != GOG_AXIS_CIRCULAR) {
+		GtkAdjustment *adj;
+		double start = axis->span_start * 100., end = axis->span_end * 100.;
+		w = go_gtk_builder_get_widget (gui, "start-btn");
+		adj = GTK_ADJUSTMENT (gtk_builder_get_object (gui, "end-adj"));
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), start);
+		gtk_adjustment_set_lower (adj, fmin (start + 1., end));
+		g_signal_connect (w, "value_changed", G_CALLBACK (cb_start_changed), axis);
+		g_object_set_data (G_OBJECT (w), "other-adj", adj);
+		w = go_gtk_builder_get_widget (gui, "end-btn");
+		adj = GTK_ADJUSTMENT (gtk_builder_get_object (gui, "start-adj"));
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), end);
+		gtk_adjustment_set_upper (adj, fmax (end - 1., start));
+		g_signal_connect (w, "value_changed", G_CALLBACK (cb_end_changed), axis);
+		g_object_set_data (G_OBJECT (w), "other-adj", adj);
+		go_editor_add_page (editor,
+		                    go_gtk_builder_get_widget (gui, "area-grid"),
+		                    _("Span"));
+	}
+
+
 	if (gog_object_is_visible (axis) && gog_axis_get_atype (axis) < GOG_AXIS_VIRTUAL) {
 	    /* Style page */
 	    (GOG_OBJECT_CLASS(parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 
 	    /* Format page */
-	    if (!axis->is_discrete) {
+	    {
 		    GOFormat *fmt = gog_axis_get_effective_format (axis);
 		    w = go_format_sel_new_full (TRUE);
 		    state->format_selector = w;
@@ -2523,7 +2657,7 @@ gog_axis_populate_editor (GogObject *gobj,
 	    }
 	}
 
-	g_object_set_data_full (gtk_builder_get_object (gui, "axis_pref_box"),
+	g_object_set_data_full (gtk_builder_get_object (gui, "axis-pref-grid"),
 				"state", state, (GDestroyNotify) gog_axis_pref_state_free);
 	g_object_unref (gui);
 
@@ -2606,6 +2740,18 @@ gog_axis_class_init (GObjectClass *gobject_klass)
 			_("Polar axis set unit"),
 			polar_units[GOG_AXIS_POLAR_UNIT_DEGREES].name,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, AXIS_PROP_SPAN_START,
+		g_param_spec_double ("span-start",
+			_("Axis start position"),
+			_("Position of the plot area at which the axis effective area starts, expressed as a percentage of the available position. Defaults to 0.0"),
+			0., 1., 0.,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, AXIS_PROP_SPAN_END,
+		g_param_spec_double ("span-end",
+			_("Axis end position"),
+			_("Position of the plot area at which the axis effective area ends, expressed as a percentage of the available position. Defaults to 1.0"),
+			0., 1., 1.,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
 
@@ -2641,6 +2787,8 @@ gog_axis_init (GogAxis *axis)
 
 	axis->ticks = NULL;
 	axis->tick_nbr = 0;
+	axis->span_start = 0.;
+	axis->span_end = 1.;
 }
 
 static void
@@ -2777,6 +2925,15 @@ gog_axis_set_bounds (GogAxis *axis, double minimum, double maximum)
 	}
 }
 
+void
+gog_axis_get_effective_span (GogAxis const *axis, double *start, double *end)
+{
+	g_return_if_fail (GOG_IS_AXIS (axis));
+
+	*start = axis->span_start;
+	*end = axis->span_end;
+}
+
 /**
  * gog_axis_set_extents :
  * @axis : #GogAxis
@@ -3203,6 +3360,7 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 	GogChart *chart = GOG_CHART (gog_object_get_parent (view->model));
 	double const pad_h = gog_renderer_pt2r_y (view->renderer, PAD_HACK);
 	double const pad_w = gog_renderer_pt2r_x (view->renderer, PAD_HACK);
+	double start, end;
 
 	available.w = bbox->w;
 	available.h = bbox->h;
@@ -3226,9 +3384,10 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 						return;
 					}
 					gog_view_size_request (child, &available, &req);
+					gog_axis_get_effective_span (axis, &start, &end);
 					if (type == GOG_AXIS_X) {
 						child_bbox.x = plot_area->x +
-							(plot_area->w - req.w) / 2.0;
+							(plot_area->w * (start + end) - req.w) / 2.0;
 						child_bbox.w = req.w;
 						child_bbox.h = req.h;
 						switch (axis_pos) {
@@ -3244,7 +3403,7 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 						tmp.h -= req.h + pad_h;
 					} else {
 						child_bbox.y = plot_area->y +
-							(plot_area->h - req.h) / 2.0;
+							(plot_area->h * (start + end) - req.h) / 2.0;
 						child_bbox.h = req.h;
 						child_bbox.w = req.w;
 						switch (axis_pos) {
