@@ -154,7 +154,7 @@ gog_series_element_populate_editor (GogObject *gobj,
 
 	(GOG_OBJECT_CLASS(gse_parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 
-	w = gtk_hbox_new (FALSE, 12);
+	w = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
 	gtk_box_pack_start (GTK_BOX (w), gtk_label_new (_("Index:")),
 			    FALSE, FALSE, 0);
 	spin_button = gtk_spin_button_new_with_range (0, G_MAXINT, 1);
@@ -165,7 +165,7 @@ gog_series_element_populate_editor (GogObject *gobj,
 			  G_CALLBACK (cb_index_changed), gobj);
 	gtk_box_pack_start(GTK_BOX (w), spin_button, FALSE, FALSE, 0);
 	if (gse_vbox == NULL) {
-		vbox = gtk_vbox_new (FALSE, 6);
+		vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 		gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 	} else
 		vbox = gse_vbox;
@@ -300,6 +300,36 @@ role_series_element_pre_remove (GogObject *parent, GogObject *child)
 	series->overrides = g_list_remove (series->overrides, child);
 }
 
+static gboolean
+role_series_labels_can_add (GogObject const *parent)
+{
+	GogSeries *series = GOG_SERIES (parent);
+
+	return (series->allowed_pos != 0 && gog_object_get_child_by_name (parent, "Data labels") == NULL);
+}
+
+static void
+role_series_labels_post_add (GogObject *parent, GogObject *child)
+{
+	GogSeries *series = GOG_SERIES (parent);
+	GogSeriesDesc const *desc;
+	unsigned i;
+	GogSeriesLabels *labels = GOG_SERIES_LABELS (child);
+	gog_series_labels_set_allowed_position (labels, series->allowed_pos);
+	gog_series_labels_set_default_position (labels, series->default_pos);
+	/* default is to show values */
+
+	/* Are there any shared dimensions */
+	desc = &series->plot->desc.series;
+	for (i = 0; i < desc->num_dim; i++)
+		if (desc->dim[i].ms_type == GOG_MS_DIM_VALUES)
+			break;
+	labels->format = (i != desc->num_dim)?
+		                            g_strdup_printf ("%%%u", i):
+		                            g_strdup ("");
+	labels->supports_percent = GOG_PLOT_GET_CLASS (series->plot)->get_percent != NULL;
+}
+
 static void
 gog_series_finalize (GObject *obj)
 {
@@ -415,14 +445,14 @@ cb_line_interpolation_changed (GtkComboBox *box, GogSeries *series)
 {
 	GtkBuilder *gui = g_object_get_data (G_OBJECT (box), "gui");
 	GtkWidget *widget = GTK_WIDGET (g_object_get_data (G_OBJECT(box), "skip-button"));
-	GtkWidget *table = go_gtk_builder_get_widget (gui, "clamps-table");
+	GtkWidget *grid = go_gtk_builder_get_widget (gui, "clamps-grid");
 	series->interpolation = gtk_combo_box_get_active (box);
 	gtk_widget_set_sensitive (widget, !go_line_interpolation_auto_skip (series->interpolation));
 	widget = GTK_WIDGET (g_object_get_data (G_OBJECT(box), "fill-type"));
 	if (series->interpolation == GO_LINE_INTERPOLATION_CLAMPED_CUBIC_SPLINE)
-		gtk_widget_show (table);
+		gtk_widget_show (grid);
 	else
-		gtk_widget_hide (table);
+		gtk_widget_hide (grid);
 	if (widget)
 		gtk_widget_set_sensitive (widget, !go_line_interpolation_auto_skip (series->interpolation));
 	gog_object_emit_changed (GOG_OBJECT (series), FALSE);
@@ -449,7 +479,7 @@ gog_series_populate_editor (GogObject *gobj,
 		   GOCmdContext *cc)
 {
 	static guint series_pref_page = 1;
-	GtkWidget *w, *box;
+	GtkWidget *w, *grid;
 	GtkTable  *table;
 	unsigned i, row = 0;
 	gboolean has_shared = FALSE;
@@ -458,7 +488,7 @@ gog_series_populate_editor (GogObject *gobj,
 	GogDataset *set = GOG_DATASET (gobj);
 	GogSeriesDesc const *desc;
 	GogDataType data_type;
-	GtkComboBox *combo = NULL;
+	GtkComboBoxText *combo = NULL;
 
 	g_return_if_fail (series->plot != NULL);
 
@@ -491,7 +521,7 @@ gog_series_populate_editor (GogObject *gobj,
 	}
 
 	if (has_shared) {
-		gtk_table_attach (table, gtk_hseparator_new (),
+		gtk_table_attach (table, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL),
 			0, 2, row, row+1, GTK_FILL, 0, 0, 0);
 		row++;
 	}
@@ -506,7 +536,7 @@ gog_series_populate_editor (GogObject *gobj,
 				desc->dim[i].name, desc->dim[i].priority, TRUE);
 	}
 
-	gtk_table_attach (table, gtk_hseparator_new (),
+	gtk_table_attach (table, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL),
 		0, 2, row, row+1, GTK_FILL, 0, 0, 0);
 	row++;
 	w = gtk_check_button_new_with_mnemonic (_("_Show in Legend"));
@@ -523,54 +553,53 @@ gog_series_populate_editor (GogObject *gobj,
 
 	(GOG_OBJECT_CLASS(series_parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 
-	box = go_editor_get_registered_widget (editor, "line_box");
-	if (series_class->has_interpolation && box != NULL) {
+	grid = go_editor_get_registered_widget (editor, "line-grid");
+	if (series_class->has_interpolation && grid != NULL) {
 		GtkBuilder *gui;
 		GtkWidget *widget;
 
-		gui = go_gtk_builder_new ("gog-series-prefs.ui", GETTEXT_PACKAGE, cc);
+		gui = go_gtk_builder_new_internal ("gog-series-prefs.ui", GETTEXT_PACKAGE, cc);
 		if (gui != NULL) {
 			int i;
 			GogAxisSet set = gog_plot_axis_set_pref (gog_series_get_plot (series));
 			GogDataset *clamp_set = gog_series_get_interpolation_params (series);
-			widget = go_gtk_builder_get_widget (gui, "interpolation_prefs");
-			gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 0);
-			widget = go_gtk_builder_get_widget (gui, "interpolation-table");
+			widget = go_gtk_builder_get_widget (gui, "interpolation-prefs");
+			gtk_grid_attach (GTK_GRID (grid), widget, 0, 3, 4, 1);
 			/* create an interpolation type combo and populate it */
-			combo = GTK_COMBO_BOX (gtk_combo_box_new_text ());
+			combo = GTK_COMBO_BOX_TEXT (gtk_combo_box_text_new ());
 			if (set & 1 << GOG_AXIS_RADIAL)
 				for (i = 0; i < GO_LINE_INTERPOLATION_MAX; i++) {
 					if (go_line_interpolation_supports_radial (i))
-						gtk_combo_box_append_text (combo, _(go_line_interpolation_as_label (i)));
+						gtk_combo_box_text_append_text (combo, _(go_line_interpolation_as_label (i)));
 				}
 			else
 				for (i = 0; i < GO_LINE_INTERPOLATION_MAX; i++)
-					gtk_combo_box_append_text (combo, _(go_line_interpolation_as_label (i)));
-			gtk_combo_box_set_active (combo, series->interpolation);
+					gtk_combo_box_text_append_text (combo, _(go_line_interpolation_as_label (i)));
+			gtk_combo_box_set_active (GTK_COMBO_BOX (combo), series->interpolation);
 			g_signal_connect (combo, "changed",
 					  G_CALLBACK (cb_line_interpolation_changed), series);
-			gtk_table_attach (GTK_TABLE (widget), GTK_WIDGET (combo), 1, 2,
-					  0, 1, (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
-					  (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0);
-			gtk_widget_show_all (widget);
-			widget = go_gtk_builder_get_widget (gui, "interpolation-skip-invalid");
-			g_object_set_data (G_OBJECT (combo), "skip-button", widget);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), series->interpolation_skip_invalid);
+			gtk_grid_attach (GTK_GRID (widget), GTK_WIDGET (combo), 1, 1, 1, 1),
+			w = go_gtk_builder_get_widget (gui, "interpolation-skip-invalid");
+			g_object_set_data (G_OBJECT (combo), "skip-button", w);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), series->interpolation_skip_invalid);
 			if (go_line_interpolation_auto_skip (series->interpolation))
-				gtk_widget_set_sensitive (widget, FALSE);
-			g_signal_connect (widget, "toggled",
+				gtk_widget_set_sensitive (w, FALSE);
+			g_signal_connect (w, "toggled",
 					  G_CALLBACK (cb_line_interpolation_skip_changed), series);
+			gtk_widget_show_all (widget);
 			if (clamp_set) {
 				GtkWidget *w;
-				widget = go_gtk_builder_get_widget (gui, "clamps-table");
+				widget = go_gtk_builder_get_widget (gui, "clamps-grid");
 				w = GTK_WIDGET (gog_data_allocator_editor (dalloc, clamp_set, 0, GOG_DATA_SCALAR));
 				gtk_widget_set_tooltip_text (w, _("Derivative at first point of the clamped cubic spline."));
 				gtk_widget_show (w);
-				gtk_table_attach (GTK_TABLE (widget), w, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+				gtk_widget_set_hexpand (w, TRUE);
+				gtk_grid_attach (GTK_GRID (widget), w, 1, 0, 2, 1);
 				w = GTK_WIDGET (gog_data_allocator_editor (dalloc, clamp_set, 1, GOG_DATA_SCALAR));
 				gtk_widget_set_tooltip_text (w, _("Derivative at last point of the clamped cubic spline."));
 				gtk_widget_show (w);
-				gtk_table_attach (GTK_TABLE (widget), w, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+				gtk_widget_set_hexpand (w, TRUE);
+				gtk_grid_attach (GTK_GRID (widget), w, 1, 1, 2, 1);
 				if (series->interpolation != GO_LINE_INTERPOLATION_CLAMPED_CUBIC_SPLINE)
 					gtk_widget_hide (widget);
 			}
@@ -579,15 +608,15 @@ gog_series_populate_editor (GogObject *gobj,
 		}
 	}
 
-	box = go_editor_get_registered_widget (editor, "fill_extension_box");
-	if (series_class->has_fill_type && box != NULL) {
+	grid = go_editor_get_registered_widget (editor, "fill-grid");
+	if (series_class->has_fill_type && grid != NULL) {
 		GtkBuilder *gui;
 		GtkWidget *widget;
 
-		gui = go_gtk_builder_new ("gog-series-prefs.ui", GETTEXT_PACKAGE, cc);
+		gui = go_gtk_builder_new_internal ("gog-series-prefs.ui", GETTEXT_PACKAGE, cc);
 		if (gui != NULL) {
 			widget = go_gtk_builder_get_widget (gui, "fill_type_combo");
-			gog_series_populate_fill_type_combo (GOG_SERIES (series), GTK_COMBO_BOX (widget));
+				gog_series_populate_fill_type_combo (GOG_SERIES (series), GTK_COMBO_BOX (widget));
 			g_signal_connect (G_OBJECT (widget), "changed",
 					  G_CALLBACK (cb_fill_type_changed), series);
 			if (combo)
@@ -595,7 +624,7 @@ gog_series_populate_editor (GogObject *gobj,
 			if (series->interpolation == GO_LINE_INTERPOLATION_CLOSED_SPLINE)
 				gtk_widget_set_sensitive (widget, FALSE);
 			widget = go_gtk_builder_get_widget (gui, "fill_type_prefs");
-			gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 0);
+			gtk_grid_attach (GTK_GRID (grid), widget, 0, 1, 3, 1);
 			g_object_set_data (G_OBJECT (widget), "gui", gui);
 
 			g_signal_connect_swapped (G_OBJECT (widget), "destroy",
@@ -611,7 +640,12 @@ static void
 gog_series_update (GogObject *obj)
 {
 	GogSeries *series = GOG_SERIES (obj);
+	GogObjectRole const *role = gog_object_find_role_by_name (obj, "Data labels");
+	GSList *l = gog_object_get_children (obj, role), *ptr;
 	series->needs_recalc = FALSE;
+	for (ptr = l; ptr; ptr = ptr->next)
+		gog_object_request_update (ptr->data);
+	g_slist_free (l);
 }
 
 static void
@@ -657,6 +691,14 @@ gog_series_class_init (GogSeriesClass *klass)
 		  regression_curve_post_add,
 		  regression_curve_pre_remove,
 		  NULL },
+		{ N_("Data labels"), "GogSeriesLabels",	3,
+		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+		  role_series_labels_can_add,
+		  NULL,
+		  NULL,
+		  role_series_labels_post_add,
+		  NULL,
+		  NULL }
 	};
 	unsigned int i;
 
@@ -724,6 +766,7 @@ gog_series_init (GogSeries *series)
 	series->index = -1;
 	series->acceptable_children = 0;
 	series->interpolation = GO_LINE_INTERPOLATION_LINEAR;
+	series->default_pos = GOG_SERIES_LABELS_CENTERED;
 }
 
 static void
@@ -981,6 +1024,33 @@ gog_series_set_dim (GogSeries *series, int dim_i, GOData *val, GError **err)
 	gog_dataset_set_dim (GOG_DATASET (series), dim_i, val, err);
 }
 
+int
+gog_series_map_XL_dim (GogSeries const *series, GogMSDimType ms_type)
+{
+	GogSeriesDesc const *desc = &series->plot->desc.series;
+	unsigned i = desc->num_dim;
+
+	if (ms_type == GOG_MS_DIM_LABELS)
+		return -1;
+	while (i-- > 0)
+		if (desc->dim[i].ms_type == ms_type)
+			return i;
+	return -2;
+}
+
+void
+gog_series_set_XL_dim (GogSeries *series, GogMSDimType ms_type, GOData *val, GError **err)
+{
+	int dim;
+	g_return_if_fail (series !=NULL);
+	dim = gog_series_map_XL_dim (series, ms_type);
+	if (dim >= -1) {
+		gog_series_set_dim (series, dim, val, err);
+		return;
+	}
+	g_object_unref (val);
+}
+
 /**
  * gog_series_num_elements :
  * @series : #GogSeries
@@ -1175,7 +1245,7 @@ gog_series_populate_fill_type_combo (GogSeries const *series, GtkComboBox *combo
 	for (i = 0; series_klass->valid_fill_type_list[i] != GOG_SERIES_FILL_TYPE_INVALID; i++) {
 		fill_type = series_klass->valid_fill_type_list[i];
 		if (fill_type >= 0 && fill_type < GOG_SERIES_FILL_TYPE_INVALID) {
-			gtk_combo_box_append_text (combo, _(_fill_type_infos[fill_type].label));
+			go_gtk_combo_box_append_text (combo, _(_fill_type_infos[fill_type].label));
 			if (fill_type == series->fill_type)
 				gtk_combo_box_set_active (combo, i);
 		}
