@@ -37,20 +37,20 @@
 /**
  * SECTION:gog-theme
  * @short_description: a list of default styles to apply to appropriate graph elements.
- * 
+ *
  * The library provides two hard coded themes, "Default", and "Guppi". Other themes
  * are described in files, some of which might be distributed with the library.
- * 
+ *
  * A file defining a theme is an xml file with a &lt;GogTheme&gt; root node. The contents
  * must be: _name|name+, _description?|description*, GOStyle+.
- * 
+ *
  * _name and name nodes:
  *
  * The _name node should be used for themes distributed with goffice, localized
  * names will be in *.po files and only the default name for "C" locale needs to
  * be there. Other files need at least one name node for the default name, and
  * might have some translated names with an appropriate "xml:lang" attribute.
- * 
+ *
  * _description and description nodes:
  *
  * These work just like name nodes. The difference is that no description node is
@@ -811,6 +811,26 @@ static void build_predefined_themes (void)
 	gog_theme_add_element (theme, style,
 		NULL, "GogRegEqn", NULL);
 
+	/* series labels */
+	style = go_style_new ();
+	style->line.dash_type = GO_LINE_NONE;
+	style->line.width = 0; /* none */
+	style->line.color = GO_COLOR_BLACK;
+	style->fill.type = GO_STYLE_FILL_NONE;
+	go_pattern_set_solid (&style->fill.pattern, GO_COLOR_WHITE);
+	go_style_set_font_desc (style, pango_font_description_from_string ("Sans 6"));
+	gog_theme_add_element (theme, style, NULL, "GogSeriesLabels", NULL);
+
+	/* data label */
+	style = go_style_new ();
+	style->line.dash_type = GO_LINE_NONE;
+	style->line.width = 0; /* none */
+	style->line.color = GO_COLOR_BLACK;
+	style->fill.type = GO_STYLE_FILL_NONE;
+	go_pattern_set_solid (&style->fill.pattern, GO_COLOR_WHITE);
+	go_style_set_font_desc (style, pango_font_description_from_string ("Sans 6"));
+	gog_theme_add_element (theme, style, NULL, "GogDataLabel", NULL);
+
 #ifdef GOFFICE_WITH_LASEM
 	/* Equations */
 	style = go_style_new ();
@@ -936,6 +956,26 @@ static void build_predefined_themes (void)
 	gog_theme_add_element (theme, style,
 		NULL, "GogRegEqn", NULL);
 
+	/* series labels */
+	style = go_style_new ();
+	style->line.dash_type = GO_LINE_NONE;
+	style->line.width = 0; /* none */
+	style->line.color = GO_COLOR_BLACK;
+	style->fill.type = GO_STYLE_FILL_NONE;
+	go_pattern_set_solid (&style->fill.pattern, GO_COLOR_WHITE);
+	go_style_set_font_desc (style, pango_font_description_from_string ("Sans 6"));
+	gog_theme_add_element (theme, style, NULL, "GogSeriesLabels", NULL);
+
+	/* data label */
+	style = go_style_new ();
+	style->line.dash_type = GO_LINE_NONE;
+	style->line.width = 0; /* none */
+	style->line.color = GO_COLOR_BLACK;
+	style->fill.type = GO_STYLE_FILL_NONE;
+	go_pattern_set_solid (&style->fill.pattern, GO_COLOR_WHITE);
+	go_style_set_font_desc (style, pango_font_description_from_string ("Sans 6"));
+	gog_theme_add_element (theme, style, NULL, "GogDataLabel", NULL);
+
 #ifdef GOFFICE_WITH_LASEM
 	/* Equations */
 	style = go_style_new ();
@@ -1026,7 +1066,7 @@ desc_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 elem_start (GsfXMLIn *xin, G_GNUC_UNUSED xmlChar const **attrs)
 {
-	struct theme_load_state	*state = (struct theme_load_state *) xin->user_state; 
+	struct theme_load_state	*state = (struct theme_load_state *) xin->user_state;
 	char const *role = NULL, *class_name = NULL;
 	GOStyle *style;
 	unsigned i;
@@ -1131,7 +1171,7 @@ _gog_themes_shutdown (void)
 		default_theme = NULL;
 	}
 
-	go_slist_free_custom (g_slist_copy (themes), g_object_unref);
+	g_slist_free_full (g_slist_copy (themes), g_object_unref);
 	g_slist_free (themes);
 	themes = NULL;
 }
