@@ -22,7 +22,6 @@
 #include <goffice/goffice-config.h>
 #include <goffice/goffice.h>
 #include <goffice/utils/go-marshalers.h>
-#include <goffice/gtk/go-gtk-compat.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
@@ -94,13 +93,30 @@ typedef enum {
 	F_LIST_LABEL,		F_LIST_SCROLL,		F_LIST,
 	F_DECIMAL_SPIN,		F_ENGINEERING_BUTTON,
 	F_SUPERSCRIPT_BUTTON,	F_SUPERSCRIPT_HIDE_1_BUTTON,
+	F_SI_BUTTON,            F_SI_CUSTOM_UNIT_BUTTON,
+	F_SI_SI_UNIT_BUTTON,	F_SI_UNIT_COMBO,
+	F_EXP_DIGITS,           F_EXP_DIGITS_LABEL,
 	F_NEGATIVE_LABEL,	F_NEGATIVE_SCROLL,	F_NEGATIVE,
 	F_DECIMAL_LABEL,	F_CODE_LABEL,
+	F_FRACTION_SEPARATE_INTEGER,
+	F_FRACTION_MIN_INTEGER_DIGITS_LABEL,
+	F_FRACTION_MIN_INTEGER_DIGITS,
+	F_FRACTION_MIN_NUMERATOR_DIGITS_LABEL,
+	F_FRACTION_SPECIFIED,
+	F_FRACTION_AUTOMATIC,
+	F_FRACTION_DENOMINATOR_LABEL,
+	F_FRACTION_DENOMINATOR,
+	F_FRACTION_MAX_DENOM_DIGITS_LABEL,
+	F_FRACTION_MIN_DENOM_DIGITS_LABEL,
+	F_FRACTION_MAX_DENOM_DIGITS,
+	F_FRACTION_MIN_DENOM_DIGITS,
+	F_FRACTION_PI_SCALE,
+	F_BASE_SEPARATOR,
 	F_MAX_WIDGET
 } FormatWidget;
 
 struct  _GOFormatSel {
-	GtkHBox   box;
+	GtkBox   box;
 	GtkBuilder *gui;
 
 	gpointer  value;
@@ -137,12 +153,14 @@ struct  _GOFormatSel {
 		GOFormat const	*spec;
 		gint		current_type;
 
+		char *default_si_unit;
+
 		GOFormatDetails details;
 	} format;
 };
 
 typedef struct {
-	GtkHBoxClass parent_class;
+	GtkBoxClass parent_class;
 
 	gboolean  (*format_changed)   (GOFormatSel *gfs, char const *fmt);
 	char     *(*generate_preview) (GOFormatSel *gfs, char *fmt);
@@ -160,12 +178,12 @@ static guint go_format_sel_signals [LAST_SIGNAL] = { 0 };
 static void format_entry_set_text (GOFormatSel *gfs, const gchar *text);
 
 static char *
-generate_preview (GOFormatSel *gfs, GOColor *c)
+generate_preview (GOFormatSel *gfs, PangoAttrList **attrs)
 {
 	char *res = NULL;
 	g_signal_emit (G_OBJECT (gfs),
 		       go_format_sel_signals [GENERATE_PREVIEW], 0,
-		       c, &res);
+		       attrs, &res);
 	return res;
 }
 
@@ -174,8 +192,7 @@ draw_format_preview (GOFormatSel *gfs, gboolean regen_format)
 {
 	char *preview;
 	GOFormat const *sf = NULL;
-	GOColor c = 0;
-	GdkColor gdk_color;
+	PangoAttrList *attrs = NULL;
 	gsize len;
 
 	if (regen_format) {
@@ -196,22 +213,23 @@ draw_format_preview (GOFormatSel *gfs, gboolean regen_format)
 	if (NULL == (sf = gfs->format.spec))
 		return;
 
-	if (NULL == (preview = generate_preview (gfs, &c)))
+	if (NULL == (preview = generate_preview (gfs, &attrs)))
 		return;
 
 	len = g_utf8_strlen (preview, -1);
 	if (len > FORMAT_PREVIEW_MAX)
-		strcpy (g_utf8_offset_to_pointer (preview, FORMAT_PREVIEW_MAX - 5),
+		strcpy (g_utf8_offset_to_pointer (preview,
+						  FORMAT_PREVIEW_MAX - 5),
 			"...");
 
 	gtk_text_buffer_set_text (gfs->format.preview_buffer, preview, -1);
-	if (c != 0)
-		go_color_to_gdk (c, &gdk_color);
-	else
-		gdk_color_parse ("black", &gdk_color);
 
-	gtk_widget_modify_text (GTK_WIDGET (gfs->format.preview),
-				GTK_STATE_NORMAL, &gdk_color);
+	go_load_pango_attributes_into_buffer (attrs,
+					      gfs->format.preview_buffer,
+					      preview);
+
+	pango_attr_list_unref (attrs);
+
 	g_free (preview);
 }
 
@@ -278,6 +296,65 @@ fillin_negative_samples (GOFormatSel *gfs)
 	END_LOCALE_SWITCH;
 }
 
+static struct {
+	char const *name;
+	char const * unit;
+	int scale;
+} si_units[] = {
+	{N_("A (ampere)"), "A", 0},
+	{N_("Bq (becquerel)"), "Bq", 0},
+	{N_("cd (candela)"), "cd", 0},
+	{N_("C (coulomb)"), "C", 0},
+	{N_("\302\260C (degree Celsius)"), "\302\260C", 0},
+	{N_("F (farad)"), "F", 0},
+	{N_("Gy (gray)"), "Gy", 0},
+	{N_("H (henry)"), "H", 0},
+	{N_("Hz (hertz)"), "Hz", 0},
+	{N_("J (joule)"), "J", 0},
+	{N_("K (kelvin)"), "K", 0},
+	{N_("kg (kilogram)"), "kg", 3},
+	{N_("lm (lumen)"), "lm", 0},
+	{N_("lx (lux)"), "lx", 0},
+	{N_("m (meter)"), "m", 0},
+	{N_("mol (mole)"), "mol", 0},
+	{N_("N (newton)"), "N", 0},
+	{N_("\316\251 (ohm)"), "\316\251", 0},
+	{N_("Pa (pascal)"), "Pa", 0},
+	{N_("rad (radian)"), "rad", 0},
+	{N_("s (second)"), "s", 0},
+	{N_("S (siemens)"), "S", 0},
+	{N_("sr (steradian)"), "sr", 0},
+	{N_("Sv (sievert)"), "Sv", 0},
+	{N_("T (tesla)"), "T", 0},
+	{N_("kat (katal)"), "kat", 0},
+	{N_("V (volt)"), "V", 0},
+	{N_("W (watt)"), "W", 0},
+	{N_("Wb (weber)"), "Wb", 0}
+};
+
+static void
+load_si_combo (GtkWidget *w, GOFormatSel *gfs)
+{
+	GtkComboBox *c = GTK_COMBO_BOX (w);
+	GtkListStore *store = GTK_LIST_STORE (gtk_combo_box_get_model (c));
+	guint i;
+	GtkCellRenderer  *cell;
+
+	for (i = 0; i < G_N_ELEMENTS (si_units); i++)
+		gtk_list_store_insert_with_values
+			(store, NULL, i,
+			 0, _(si_units[i].name),
+			 1, si_units[i].unit,
+			 2, si_units[i].scale,
+			 -1);
+	cell = gtk_cell_renderer_text_new();
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(c), cell, TRUE);
+	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(c), cell, "text",
+				       0, NULL);
+	gtk_combo_box_set_id_column (c, 1);
+}
+
+
 static void
 cb_decimals_changed (GtkSpinButton *spin, GOFormatSel *gfs)
 {
@@ -290,6 +367,62 @@ cb_decimals_changed (GtkSpinButton *spin, GOFormatSel *gfs)
 	draw_format_preview (gfs, TRUE);
 }
 
+static void
+cb_denominator_changed (GtkSpinButton *spin, GOFormatSel *gfs)
+{
+	gfs->format.details.denominator =
+		gtk_spin_button_get_value_as_int (spin);
+
+	draw_format_preview (gfs, TRUE);
+}
+
+static void
+cb_max_denom_digits_changed (GtkSpinButton *spin, GOFormatSel *gfs)
+{
+	int val = gtk_spin_button_get_value_as_int (spin);
+
+	gfs->format.details.denominator_max_digits = val;
+	gtk_spin_button_set_range
+		 (GTK_SPIN_BUTTON (gfs->format.widget[F_FRACTION_MIN_DENOM_DIGITS]),
+		  0, val);
+
+	draw_format_preview (gfs, TRUE);
+}
+
+static void
+cb_min_integer_digits_changed (GtkSpinButton *spin, GOFormatSel *gfs)
+{
+	gfs->format.details.min_digits =
+		gtk_spin_button_get_value_as_int (spin);
+
+	draw_format_preview (gfs, TRUE);
+}
+
+static void
+cb_min_denom_digits_changed (GtkSpinButton *spin, GOFormatSel *gfs)
+{
+	gfs->format.details.denominator_min_digits =
+		gtk_spin_button_get_value_as_int (spin);
+
+	draw_format_preview (gfs, TRUE);
+}
+
+static void
+cb_exp_digits_changed (GtkSpinButton *spin, GOFormatSel *gfs)
+{
+	int val = gtk_spin_button_get_value_as_int (spin);
+
+	if (gfs->format.current_type == GO_FORMAT_FRACTION)
+		gfs->format.details.numerator_min_digits = val;
+	else {
+		gfs->format.details.exponent_digits = val;
+		if (gtk_widget_get_visible (gfs->format.widget[F_NEGATIVE]))
+			fillin_negative_samples (gfs);
+	}
+
+	draw_format_preview (gfs, TRUE);
+}
+
 static void
 cb_separator_toggle (GtkWidget *w, GOFormatSel *gfs)
 {
@@ -313,6 +446,53 @@ cb_engineering_toggle (GtkWidget *w, GOFormatSel *gfs)
 	draw_format_preview (gfs, TRUE);
 }
 
+static void
+cb_si_toggle (GtkWidget *w, GOFormatSel *gfs)
+{
+	gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
+
+	gfs->format.details.append_SI = active;
+
+	gtk_widget_set_sensitive (gfs->format.widget[F_SI_CUSTOM_UNIT_BUTTON],
+				  active);
+	gtk_widget_set_sensitive (gfs->format.widget[F_SI_SI_UNIT_BUTTON],
+				  active);
+	gtk_widget_set_sensitive (gfs->format.widget[F_SI_UNIT_COMBO],
+				  active);
+
+	draw_format_preview (gfs, TRUE);
+}
+
+static void
+cb_si_unit_toggle (GtkWidget *w, GOFormatSel *gfs)
+{
+	g_free (gfs->format.details.appended_SI_unit);
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w))) {
+		GtkComboBox *c = GTK_COMBO_BOX (gfs->format.widget[F_SI_UNIT_COMBO]);
+		GtkTreeIter iter;
+		GtkTreeModel *store = gtk_combo_box_get_model (c);
+		if (gtk_combo_box_get_active_iter (c, &iter)) {
+			gtk_tree_model_get (store, &iter, 1, &gfs->format.details.appended_SI_unit,
+					    2, &gfs->format.details.scale, -1);
+			if (gfs->format.details.scale == 3) {
+				char *text = gfs->format.details.appended_SI_unit;
+				gfs->format.details.appended_SI_unit = g_strdup (text + 1);
+				g_free (text);
+			}
+		} else
+			gfs->format.details.appended_SI_unit = NULL;
+	} else
+		gfs->format.details.appended_SI_unit = g_strdup (gfs->format.default_si_unit);
+
+	draw_format_preview (gfs, TRUE);
+}
+
+static void
+cb_si_combo_changed (GtkWidget *w, GOFormatSel *gfs)
+{
+	cb_si_unit_toggle (gfs->format.widget[F_SI_SI_UNIT_BUTTON], gfs);
+}
+
 static void
 cb_superscript_toggle (GtkWidget *w, GOFormatSel *gfs)
 {
@@ -333,9 +513,48 @@ cb_superscript_hide_1_toggle (GtkWidget *w, GOFormatSel *gfs)
 	draw_format_preview (gfs, TRUE);
 }
 
+static void
+cb_split_fraction_toggle (GtkWidget *w, GOFormatSel *gfs)
+{
+	gboolean act = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
+	gfs->format.details.split_fraction = act;
+	gtk_widget_set_sensitive (gfs->format.widget[F_FRACTION_MIN_INTEGER_DIGITS_LABEL],
+				  act);
+	gtk_widget_set_sensitive (gfs->format.widget[F_FRACTION_MIN_INTEGER_DIGITS],
+				  act);
+
+	draw_format_preview (gfs, TRUE);
+}
+
+static void
+cb_pi_scale_toggle (GtkWidget *w, GOFormatSel *gfs)
+{
+	gboolean act = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
+	gfs->format.details.pi_scale = act;
+
+	draw_format_preview (gfs, TRUE);
+}
+
+static void
+cb_fraction_automatic_toggle (GtkWidget *w, GOFormatSel *gfs)
+{
+	gboolean act =gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
+	gfs->format.details.automatic_denominator = act;
+	gtk_widget_set_sensitive (gfs->format.widget[F_FRACTION_MAX_DENOM_DIGITS_LABEL],
+				  act);
+	gtk_widget_set_sensitive (gfs->format.widget[F_FRACTION_MIN_DENOM_DIGITS_LABEL],
+				  act);
+	gtk_widget_set_sensitive (gfs->format.widget[F_FRACTION_MAX_DENOM_DIGITS], act);
+	gtk_widget_set_sensitive (gfs->format.widget[F_FRACTION_MIN_DENOM_DIGITS], act);
+	gtk_widget_set_sensitive (gfs->format.widget[F_FRACTION_DENOMINATOR_LABEL], !act);
+	gtk_widget_set_sensitive (gfs->format.widget[F_FRACTION_DENOMINATOR], !act);
+
+	draw_format_preview (gfs, TRUE);
+}
+
 static void
 fmt_dialog_init_fmt_list (GOFormatSel *gfs, char const *const *formats,
-			  GtkTreeIter *select)
+			  GtkTreeIter *select, GHashTable *fhash)
 {
 	GOFormatMagic cur_magic = go_format_get_magic (gfs->format.spec);
 
@@ -350,6 +569,10 @@ fmt_dialog_init_fmt_list (GOFormatSel *gfs, char const *const *formats,
 		gtk_list_store_set (gfs->format.formats.model, &iter,
 				    0, fmt, -1);
 		g_free (fmt);
+		if (fhash)
+			g_hash_table_insert (fhash,
+					     (gpointer)(*formats),
+					     GINT_TO_POINTER (1));
 
 		/* Magic formats are fully defined by their magic.  */
 		found = cur_magic
@@ -393,12 +616,40 @@ find_builtin (const char *fmtstr, int page, gboolean def)
 	return candidates[list_elem];
 }
 
+typedef struct {
+	GOFormatSel *gfs;
+	GtkTreeIter *select;
+	GHashTable  *hash;
+} fmt_dialog_closure_t;
+
+static void
+fmt_dialog_load_true_custom_cb (char const *key, GOFormat const *fmt,
+				fmt_dialog_closure_t *cl)
+{
+	GtkTreeIter iter;
+	char *fmt_string;
+	if (cl->hash != NULL && NULL != g_hash_table_lookup (cl->hash, key))
+		return;
+	fmt_string = go_format_str_localize (key);
+	gtk_list_store_insert_with_values (cl->gfs->format.formats.model, &iter,
+					   0, 0, fmt_string, -1);
+	g_free (fmt_string);
+	if (cl->select->stamp == 0) {
+		GOFormatMagic cur_magic = go_format_get_magic (cl->gfs->format.spec);
+		GOFormatMagic magic = go_format_get_magic (fmt);
+		gboolean found = cur_magic
+			? (cur_magic == magic)
+			: go_format_eq (fmt, cl->gfs->format.spec);
+		if (found)
+			*(cl->select) = iter;
+	}
+}
 
 static void
 fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 {
 	SETUP_LOCALE_SWITCH;
-	static FormatWidget const contents[][12] = {
+	static FormatWidget const contents[][17] = {
 		/* General */
 		{
 			F_GENERAL_EXPLANATION,
@@ -467,9 +718,23 @@ fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 		/* Fraction */
 		{
 			F_FRACTION_EXPLANATION,
-			F_LIST_LABEL,
-			F_LIST_SCROLL,
-			F_LIST,
+			F_FRACTION_SEPARATE_INTEGER,
+			F_FRACTION_MIN_INTEGER_DIGITS_LABEL,
+			F_FRACTION_MIN_INTEGER_DIGITS,
+			F_FRACTION_MIN_NUMERATOR_DIGITS_LABEL,
+			F_EXP_DIGITS,
+			F_FRACTION_SPECIFIED,
+			F_FRACTION_AUTOMATIC,
+			F_FRACTION_DENOMINATOR_LABEL,
+			F_FRACTION_DENOMINATOR,
+			F_FRACTION_MAX_DENOM_DIGITS_LABEL,
+			F_FRACTION_MIN_DENOM_DIGITS_LABEL,
+			F_FRACTION_MAX_DENOM_DIGITS,
+			F_FRACTION_MIN_DENOM_DIGITS,
+#ifdef ALLOW_PI_SLASH
+			F_FRACTION_PI_SCALE,
+			F_BASE_SEPARATOR,
+#endif
 			F_MAX_WIDGET
 		},
 		/* Scientific */
@@ -480,6 +745,14 @@ fmt_dialog_enable_widgets (GOFormatSel *gfs, int page)
 			F_ENGINEERING_BUTTON,
 			F_SUPERSCRIPT_BUTTON,
 			F_SUPERSCRIPT_HIDE_1_BUTTON,
+			F_EXP_DIGITS,
+			F_EXP_DIGITS_LABEL,
+#ifdef ALLOW_SI_APPEND
+			F_SI_BUTTON,
+			F_SI_CUSTOM_UNIT_BUTTON,
+			F_SI_SI_UNIT_BUTTON,
+			F_SI_UNIT_COMBO,
+#endif
 			F_MAX_WIDGET
 		},
 		/* Text */
@@ -547,13 +820,16 @@ stays:
 		case F_LIST: {
 			int start = 0, end = -1;
 			GtkTreeIter select;
+			GHashTable *fhash = NULL;
+
+			if  (page == FMT_CUSTOM)
+				fhash = g_hash_table_new (g_str_hash, g_str_equal);
 
 			switch (page) {
 			default :
 				;
 			case GO_FORMAT_DATE:
 			case GO_FORMAT_TIME:
-			case GO_FORMAT_FRACTION:
 				start = end = page;
 				break;
 
@@ -565,15 +841,19 @@ stays:
 			select.stamp = 0;
 			gtk_list_store_clear (gfs->format.formats.model);
 			for (; start <= end ; ++start)
-				fmt_dialog_init_fmt_list (gfs,
-					_go_format_builtins[start], &select);
+				fmt_dialog_init_fmt_list (gfs, _go_format_builtins[start],
+							  &select, fhash);
+
+			if  (page == FMT_CUSTOM) {
+				fmt_dialog_closure_t cl = {gfs, &select, fhash};
+				go_format_foreach ((GHFunc)fmt_dialog_load_true_custom_cb, &cl);
+			}
+
+			if (fhash)
+				g_hash_table_unref (fhash);
 
 			/* If this is the custom page and the format has
 			* not been found append it */
-			/* TODO We should add the list of other custom formats created.
-			*      It should be easy.  All that is needed is a way to differentiate
-			*      the std formats and the custom formats in the GOFormat hash.
-			*/
 			if  (page == FMT_CUSTOM && select.stamp == 0) {
 				const char *fmtstr = go_format_as_XL (gfs->format.spec);
 				char *tmp = go_format_str_localize (fmtstr);
@@ -609,37 +889,203 @@ stays:
 			break;
 
 		case F_DECIMAL_SPIN:
-			gtk_spin_button_set_value (GTK_SPIN_BUTTON (gfs->format.widget[F_DECIMAL_SPIN]),
+			gtk_spin_button_set_value (GTK_SPIN_BUTTON (w),
 						   gfs->format.details.num_decimals);
 			break;
 
 		case F_SUPERSCRIPT_BUTTON:
+#ifdef ALLOW_EE_MARKUP
 			if (gfs->show_format_with_markup) {
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
 							      gfs->format.details.use_markup);
 			} else
+#endif
 				show_widget = FALSE;
 			break;
 
 		case F_SUPERSCRIPT_HIDE_1_BUTTON:
+#ifdef ALLOW_EE_MARKUP
 			if (gfs->show_format_with_markup) {
 
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
 							      gfs->format.details.simplify_mantissa);
 				gtk_widget_set_sensitive (w, gfs->format.details.use_markup);
 			} else
+#endif
 				show_widget = FALSE;
 			break;
 
+		case F_EXP_DIGITS:
+			if (page == GO_FORMAT_FRACTION) {
+				gtk_spin_button_set_range (GTK_SPIN_BUTTON (w), 0, 30);
+				gtk_spin_button_set_value
+					(GTK_SPIN_BUTTON (w),
+					 gfs->format.details.numerator_min_digits);
+			} else {
+				gtk_spin_button_set_range (GTK_SPIN_BUTTON (w), 1, 10);
+				gtk_spin_button_set_value
+					(GTK_SPIN_BUTTON (w),
+					 gfs->format.details.exponent_digits);
+			}
+			break;
+
+		case F_SI_BUTTON:
+#ifdef ALLOW_SI_APPEND
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON (w),
+				 gfs->format.details.append_SI);
+#else
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON (w), FALSE);
+			show_widget = FALSE;
+#endif
+			break;
+
+		case F_SI_CUSTOM_UNIT_BUTTON:
+#ifdef ALLOW_SI_APPEND
+			/* This is set through F_SI_UNIT_COMBO */
+#else
+				show_widget = FALSE;
+#endif
+			break;
+
+		case F_SI_SI_UNIT_BUTTON:
+#ifdef ALLOW_SI_APPEND
+			/* This is set through F_SI_UNIT_COMBO */
+#else
+				show_widget = FALSE;
+#endif
+			break;
+
+		case F_SI_UNIT_COMBO: {
+#ifdef ALLOW_SI_APPEND
+			gint row = -1;
+			guint ii;
+
+			if (gfs->format.details.appended_SI_unit != NULL) {
+				char const *unit = gfs->format.details.appended_SI_unit;
+				if (unit[0] == 'g' && unit[1] == 0)
+					unit = "kg";
+				for (ii = 0; ii < G_N_ELEMENTS (si_units); ii++)
+					if (0 == strcmp (si_units[ii].unit, unit)) {
+						row = (gint)ii;
+						break;
+					}
+			}
+
+			if (row == -1) {
+				if (gfs->format.details.appended_SI_unit == NULL) {
+					gtk_button_set_label
+						(GTK_BUTTON (gfs->format.widget[F_SI_CUSTOM_UNIT_BUTTON]),
+						 _("Append no further unit."));
+					g_free (gfs->format.default_si_unit);
+					gfs->format.default_si_unit = NULL;
+				} else {
+					gchar *label;
+					gfs->format.default_si_unit
+						= g_strdup (gfs->format.details.appended_SI_unit);
+					label = g_strdup_printf (_("Append \'%s\'."),
+								 gfs->format.default_si_unit);
+					gtk_button_set_label
+						(GTK_BUTTON (gfs->format.widget[F_SI_CUSTOM_UNIT_BUTTON]),
+						 label);
+					g_free (label);
+				}
+			} else
+				gtk_combo_box_set_active_id (GTK_COMBO_BOX (w), si_units[row].unit);
+
+			if (row >= 0)
+				gtk_toggle_button_set_active
+					(GTK_TOGGLE_BUTTON (gfs->format.widget[F_SI_SI_UNIT_BUTTON]),
+					 TRUE);
+			else
+				gtk_toggle_button_set_active
+					(GTK_TOGGLE_BUTTON (gfs->format.widget[F_SI_CUSTOM_UNIT_BUTTON]),
+					 TRUE);
+
+
+#else
+				show_widget = FALSE;
+#endif
+			break;
+		}
+
 		case F_ENGINEERING_BUTTON:
 			gtk_toggle_button_set_active
-				(GTK_TOGGLE_BUTTON (gfs->format.widget[F_ENGINEERING_BUTTON]),
+				(GTK_TOGGLE_BUTTON (w),
 				 gfs->format.details.exponent_step == 3);
 			break;
 
 		case F_SEPARATOR:
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gfs->format.widget[F_SEPARATOR]),
-						      gfs->format.details.thousands_sep);
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON (w),
+				 gfs->format.details.thousands_sep);
+			break;
+
+		case F_FRACTION_SEPARATE_INTEGER:
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON (w),
+				 gfs->format.details.split_fraction);
+			break;
+
+		case F_FRACTION_MIN_INTEGER_DIGITS:
+			gtk_spin_button_set_value
+				(GTK_SPIN_BUTTON (w),
+				 gfs->format.details.min_digits);
+			gtk_widget_set_sensitive
+				(w, gfs->format.details.split_fraction);
+			gtk_widget_set_sensitive
+				(gfs->format.widget[F_FRACTION_MIN_INTEGER_DIGITS_LABEL],
+				 gfs->format.details.split_fraction);
+			break;
+
+		case F_FRACTION_AUTOMATIC:
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON (w),
+				 gfs->format.details.automatic_denominator);
+			break;
+
+		case F_FRACTION_DENOMINATOR:
+			gtk_spin_button_set_value
+				(GTK_SPIN_BUTTON (w),
+				 gfs->format.details.denominator);
+			gtk_widget_set_sensitive
+				(w, !gfs->format.details.automatic_denominator);
+			gtk_widget_set_sensitive
+				(gfs->format.widget[F_FRACTION_DENOMINATOR_LABEL],
+				 !gfs->format.details.automatic_denominator);
+			break;
+
+		case F_FRACTION_MAX_DENOM_DIGITS:
+			gtk_spin_button_set_value
+				(GTK_SPIN_BUTTON (w),
+				 gfs->format.details.denominator_max_digits);
+			gtk_widget_set_sensitive
+				(w, gfs->format.details.automatic_denominator);
+			gtk_widget_set_sensitive
+				(gfs->format.widget[F_FRACTION_MAX_DENOM_DIGITS_LABEL],
+				 gfs->format.details.automatic_denominator);
+			break;
+
+		case F_FRACTION_MIN_DENOM_DIGITS:
+			gtk_spin_button_set_value
+				(GTK_SPIN_BUTTON (w),
+				 gfs->format.details.denominator_min_digits);
+			gtk_widget_set_sensitive
+				(w, gfs->format.details.automatic_denominator);
+			gtk_widget_set_sensitive
+				(gfs->format.widget[F_FRACTION_MIN_DENOM_DIGITS_LABEL],
+				 gfs->format.details.automatic_denominator);
+			break;
+
+		case F_FRACTION_PI_SCALE:
+			gtk_toggle_button_set_active
+#ifdef ALLOW_PI_SLASH
+				(GTK_TOGGLE_BUTTON (w),
+				 gfs->format.details.pi_scale);
+#else
+				(GTK_TOGGLE_BUTTON (w), FALSE);
+#endif
 			break;
 
 		default:
@@ -726,7 +1172,7 @@ format_entry_set_text (GOFormatSel *gfs, const gchar *text)
 	GtkEntry *entry = GTK_ENTRY (gfs->format.widget[F_ENTRY]);
 
 	g_signal_handler_block (entry, gfs->format.entry_changed_id);
-	gtk_entry_set_text (entry, text);
+	gtk_entry_set_text (entry, text ? text : "");
 	g_signal_handler_unblock (entry, gfs->format.entry_changed_id);
 	cb_format_entry_changed (GTK_EDITABLE (entry), gfs);
 }
@@ -843,7 +1289,8 @@ study_format (GOFormat const *fmt, GOFormatDetails *details)
 		    !details->thousands_sep)
 			exact = FALSE;
 
-		if (details->min_digits != 1)
+		if ((details->family != GO_FORMAT_FRACTION)
+		    && (details->min_digits > 1))
 			exact = FALSE;
 	}
 
@@ -851,8 +1298,8 @@ study_format (GOFormat const *fmt, GOFormatDetails *details)
 		const char *str = go_format_as_XL (fmt);
 		if (!find_builtin (str, details->family, FALSE))
 			details->family = FMT_CUSTOM;
-	}	
-	
+	}
+
 	return details->family;
 }
 
@@ -942,11 +1389,32 @@ nfs_init (GOFormatSel *gfs)
 		"format_engineering_button",
 		"format_superscript_button",
 		"format_superscript_hide_1_button",
+		"format_SI_button",
+		"format_SI_custom_unit_button",
+		"format_SI_SI_unit_button",
+		"format_SI_unit_combo",
+		"format_exp_digits",
+		"format_exp_digits_label",
 		"format_negatives_label",
 		"format_negatives_scroll",
 		"format_negatives",
 		"format_decimal_label",
 		"format_code_label",
+		"format_separate_integer_part",
+		"format_minimum_integer_digits_label",
+		"format_minimum_integer_digits",
+		"format_minimum_numerator_digits_label",
+		"format_fraction_specified_button",
+		"format_fraction_automatic_button",
+		"format_denominator_label",
+		"format_denominator",
+		"format_max_denominator_digits_label",
+		"format_min_denominator_digits_label",
+		"format_maximum_denominator_digits",
+		"format_minimum_denominator_digits",
+		"format_pi_scale_check",
+
+		"format_base_separator",
 		NULL
 	};
 
@@ -963,7 +1431,7 @@ nfs_init (GOFormatSel *gfs)
 	gfs->show_format_with_markup = FALSE;
 	gfs->locale = NULL;
 
-	gfs->gui = go_gtk_builder_new ("go-format-sel.ui", GETTEXT_PACKAGE, NULL);
+	gfs->gui = go_gtk_builder_new_internal ("go-format-sel.ui", GETTEXT_PACKAGE, NULL);
 	if (gfs->gui == NULL)
 		return;
 
@@ -974,6 +1442,7 @@ nfs_init (GOFormatSel *gfs)
 	go_format_ref (gfs->format.spec);
 
 	gfs->format.preview = NULL;
+	gfs->format.default_si_unit = NULL;
 
 	/* The handlers will set the format family later.  -1 flags that
 	 * all widgets are already hidden. */
@@ -992,13 +1461,14 @@ nfs_init (GOFormatSel *gfs)
 		/* request width in number of chars */
 		context = gtk_widget_get_pango_context (w);
 		metrics = pango_context_get_metrics (context,
-						     gtk_widget_get_style(w)->font_desc,
+						     gtk_style_context_get_font (gtk_widget_get_style_context (w), GTK_STATE_NORMAL),
 						     pango_context_get_language (context));
 		char_width = pango_font_metrics_get_approximate_char_width (metrics);
 		gtk_widget_set_size_request (w, PANGO_PIXELS (char_width) * FORMAT_PREVIEW_MAX, -1);
 		pango_font_metrics_unref (metrics);
 	}
 	gfs->format.preview_buffer = gtk_text_view_get_buffer (gfs->format.preview);
+	go_create_std_tags_for_buffer (gfs->format.preview_buffer);
 
 	gfs->format.menu = go_gtk_builder_get_widget (gfs->gui, "format_menu");
 	populate_menu (gfs);
@@ -1063,14 +1533,43 @@ nfs_init (GOFormatSel *gfs)
 		G_CALLBACK (cb_format_negative_form_selected), gfs);
 	g_signal_connect (G_OBJECT (gfs->format.widget[F_DECIMAL_SPIN]), "value_changed",
 		G_CALLBACK (cb_decimals_changed), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_EXP_DIGITS]), "value_changed",
+		G_CALLBACK (cb_exp_digits_changed), gfs);
 	g_signal_connect (G_OBJECT (gfs->format.widget[F_SEPARATOR]), "toggled",
 		G_CALLBACK (cb_separator_toggle), gfs);
 	g_signal_connect (G_OBJECT (gfs->format.widget[F_ENGINEERING_BUTTON]), "toggled",
 		G_CALLBACK (cb_engineering_toggle), gfs);
 	g_signal_connect (G_OBJECT (gfs->format.widget[F_SUPERSCRIPT_BUTTON]), "toggled",
 		G_CALLBACK (cb_superscript_toggle), gfs);
-	g_signal_connect (G_OBJECT (gfs->format.widget[F_SUPERSCRIPT_HIDE_1_BUTTON]), "toggled",
-		G_CALLBACK (cb_superscript_hide_1_toggle), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_SI_BUTTON]), "toggled",
+		G_CALLBACK (cb_si_toggle), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_SI_SI_UNIT_BUTTON]),
+			  "toggled", G_CALLBACK (cb_si_unit_toggle), gfs);
+	load_si_combo (gfs->format.widget[F_SI_UNIT_COMBO], gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_SI_UNIT_COMBO]),
+			  "changed", G_CALLBACK (cb_si_combo_changed), gfs);
+
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_SUPERSCRIPT_HIDE_1_BUTTON]),
+			  "toggled", G_CALLBACK (cb_superscript_hide_1_toggle), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_FRACTION_AUTOMATIC]),
+			  "toggled", G_CALLBACK (cb_fraction_automatic_toggle), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_FRACTION_SEPARATE_INTEGER]),
+			  "toggled", G_CALLBACK (cb_split_fraction_toggle), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_FRACTION_PI_SCALE]),
+			  "toggled",
+			  G_CALLBACK (cb_pi_scale_toggle), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_FRACTION_MIN_INTEGER_DIGITS]),
+			  "value_changed",
+			  G_CALLBACK (cb_min_integer_digits_changed), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_FRACTION_DENOMINATOR]),
+			  "value_changed",
+			  G_CALLBACK (cb_denominator_changed), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_FRACTION_MAX_DENOM_DIGITS]),
+			  "value_changed",
+			  G_CALLBACK (cb_max_denom_digits_changed), gfs);
+	g_signal_connect (G_OBJECT (gfs->format.widget[F_FRACTION_MIN_DENOM_DIGITS]),
+			  "value_changed",
+			  G_CALLBACK (cb_min_denom_digits_changed), gfs);
 
 	/* setup custom format list */
 	gfs->format.formats.model = gtk_list_store_new (1, G_TYPE_STRING);
@@ -1148,6 +1647,9 @@ go_format_sel_finalize (GObject *obj)
 	go_format_unref (gfs->format.spec);
 	gfs->format.spec = NULL;
 
+	g_free (gfs->format.default_si_unit);
+	gfs->format.default_si_unit = NULL;
+
 	if (gfs->format.size_group) {
 		g_object_unref (gfs->format.size_group);
 		gfs->format.size_group = NULL;
@@ -1158,7 +1660,9 @@ go_format_sel_finalize (GObject *obj)
 		gfs->gui = NULL;
 	}
 
-	G_OBJECT_CLASS (g_type_class_peek (GTK_TYPE_HBOX))->finalize (obj);
+	go_format_details_finalize (&gfs->format.details);
+
+	G_OBJECT_CLASS (g_type_class_peek (GTK_TYPE_BOX))->finalize (obj);
 }
 
 static gboolean
@@ -1200,7 +1704,7 @@ nfs_class_init (GObjectClass *klass)
 }
 
 GSF_CLASS (GOFormatSel, go_format_sel,
-	   nfs_class_init, nfs_init, GTK_TYPE_HBOX)
+	   nfs_class_init, nfs_init, GTK_TYPE_BOX)
 
 /**
  * go_format_sel_new_full:
