@@ -22,15 +22,16 @@
 #include <goffice/goffice-config.h>
 #include <goffice/goffice.h>
 #include <goffice/goffice-priv.h>
-#include <goffice/gtk/go-gtk-compat.h>
 
 #include <gdk/gdkkeysyms.h>
+
 #include <atk/atkrelation.h>
 #include <atk/atkrelationset.h>
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 #include <gsf/gsf-input-stdio.h>
 #include <gsf/gsf-input-textline.h>
+#include <gsf/gsf-input-memory.h>
 
 #include <string.h>
 #ifdef HAVE_UNISTD_H
@@ -41,6 +42,10 @@
 #define PREVIEW_HSIZE 150
 #define PREVIEW_VSIZE 150
 
+#ifndef GDK_KEY_Escape
+#       define GDK_KEY_Escape GDK_Escape
+#endif
+
 /* ------------------------------------------------------------------------- */
 
 
@@ -99,6 +104,38 @@ go_gtk_dialog_add_button (GtkDialog *dialog, char const* text, char const* stock
 	return button;
 }
 
+
+static gboolean
+apply_ui_from_file (GtkBuilder *gui, GsfInput *src, const char *uifile,
+		    GError **error)
+{
+	gboolean res;
+	GsfInput *orig_src;
+
+	if (!src)
+		return FALSE;
+
+	orig_src = g_object_ref (src);
+	src = gsf_input_uncompress (src);
+	if (uifile && src == orig_src) {
+		/*
+		 * This is sad, but see bugs 662503 and 662679.  Since
+		 * ui files can contain relative filenames we must use
+		 * the _file interface to load such files.  Do no
+		 * compress such files for the time being.
+		 */
+		res = gtk_builder_add_from_file (gui, uifile, error);
+	} else {
+		size_t size = gsf_input_size (src);
+		gconstpointer data = gsf_input_read (src, size, NULL);
+		res = gtk_builder_add_from_string (gui, data, size, error);
+	}
+	g_object_unref (src);
+	g_object_unref (orig_src);
+	return res;
+}
+
+
 /**
  * go_gtk_builder_new :
  * @uifile : the name of the file load
@@ -108,45 +145,102 @@ go_gtk_dialog_add_button (GtkDialog *dialog, char const* text, char const* stock
  * Simple utility to open ui files
  *
  * Returns: a new #GtkBuilder or NULL
- **/
+ *
+ * @uifile should be one of these:
+ *
+ * res:NAME  -- data from resource manager
+ * data:DATA -- data right here
+ * filename  -- data from local file
+ *
+ * Data may be compressed, regardless of source.
+**/
 GtkBuilder *
 go_gtk_builder_new (char const *uifile,
 		    char const *domain, GOCmdContext *gcc)
 {
 	GtkBuilder *gui;
-	char *f;
 	GError *error = NULL;
+	gboolean ok = FALSE;
+	gboolean need_grid_update = (gtk_check_version (3, 2, 0) != NULL); /* remove when we require 3.2.0 or later */
 
 	g_return_val_if_fail (uifile != NULL, NULL);
 
-	if (!g_path_is_absolute (uifile))
-		f = g_build_filename (go_sys_data_dir (), "ui", uifile, NULL);
-	else
-		f = g_strdup (uifile);
-
 	gui = gtk_builder_new ();
 	if (domain)
 		gtk_builder_set_translation_domain (gui, domain);
-	if (!gtk_builder_add_from_file (gui, f, &error)) {
+
+	if (strncmp (uifile, "res:", 4) == 0) {
+		size_t len;
+		gconstpointer data = go_rsm_lookup (uifile + 4, &len);
+		GsfInput *src = data
+			? gsf_input_memory_new (data, len, FALSE)
+			: NULL;
+		ok = apply_ui_from_file (gui, src, NULL, &error);
+	} else if (strncmp (uifile, "data:", 5) == 0) {
+		const char *data = uifile + 5;
+		GsfInput *src = gsf_input_memory_new (data, strlen (data), FALSE);
+		ok = apply_ui_from_file (gui, src, NULL, &error);
+	} else {
+		/* we need to set the current directory so that the builder can find pixbufs */
+		GsfInput *src = gsf_input_stdio_new (uifile, &error);
+		ok = apply_ui_from_file (gui, src, uifile, &error);
+	}
+
+	if (!ok) {
 		g_object_unref (gui);
 		gui = NULL;
 	}
+
 	if (gui == NULL && gcc != NULL) {
 		char *msg;
 		if (error) {
 			msg = g_strdup (error->message);
 			g_error_free (error);
 		} else
-			msg = g_strdup_printf (_("Unable to open file '%s'"), f);
+			msg = g_strdup_printf (_("Unable to open file '%s'"), uifile);
 		go_cmd_context_error_system (gcc, msg);
 		g_free (msg);
 	} else if (error)
 		g_error_free (error);
-	g_free (f);
 
+	if (need_grid_update && gui) { /* remove when we require gtk+-3.2.0 or later */
+		GSList *l = gtk_builder_get_objects (gui), *ptr;
+		int rsep, csep;
+		ptr = l;
+		while (ptr) {
+			if (GTK_IS_GRID (ptr->data)) {
+				g_object_get (ptr->data, "row-spacing", &csep, "column-spacing", &rsep, NULL);
+				g_object_set (ptr->data, "row-spacing", rsep, "column-spacing", csep, NULL);
+			}
+			ptr = ptr->next;
+		}
+		g_slist_free (l);
+	}
 	return gui;
 }
 
+/*
+ * Variant of go_gtk_builder_new that searchs goffice directories
+ * for files.
+ */
+GtkBuilder *
+go_gtk_builder_new_internal (char const *uifile,
+			     char const *domain, GOCmdContext *gcc)
+{
+	char *f;
+	GtkBuilder *res;
+
+	if (g_path_is_absolute (uifile) || strncmp (uifile, "res:", 4) == 0)
+		return go_gtk_builder_new (uifile, domain, gcc);
+
+	f = g_build_filename (go_sys_data_dir (), "ui", uifile, NULL);
+	res = go_gtk_builder_new (f, domain, gcc);
+	g_free (f);
+
+	return res;
+}
+
+
 /**
  * go_gtk_builder_signal_connect :
  * @gui : #GtkBuilder
@@ -251,6 +345,38 @@ go_gtk_builder_combo_box_init_text (GtkBuilder *gui, char const *widget_name)
 	return box;
 }
 
+void
+go_gtk_combo_box_append_text (GtkComboBox *combo, char const *str)
+{
+	GtkListStore *model;
+	GtkTreeIter iter;
+
+	g_return_if_fail (GTK_IS_COMBO_BOX (combo));
+	g_return_if_fail (str != NULL);
+
+	model = GTK_LIST_STORE (gtk_combo_box_get_model (combo));
+	gtk_list_store_append (model, &iter);
+	gtk_list_store_set (model, &iter, 0, str, -1);
+}
+
+void
+go_gtk_combo_box_remove_text (GtkComboBox *combo, int position)
+{
+	GtkTreeModel *model;
+	GtkListStore *store;
+	GtkTreeIter iter;
+
+	g_return_if_fail (GTK_IS_COMBO_BOX_TEXT (combo));
+	g_return_if_fail (position >= 0);
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+	store = GTK_LIST_STORE (model);
+	g_return_if_fail (GTK_IS_LIST_STORE (store));
+
+	if (gtk_tree_model_iter_nth_child (model, &iter, NULL, position))
+		gtk_list_store_remove (store, &iter);
+}
+
 int
 go_gtk_builder_group_value (GtkBuilder *gui, char const * const group[])
 {
@@ -382,7 +508,7 @@ go_gtk_window_set_transient (GtkWindow *toplevel, GtkWindow *window)
 static gint
 cb_non_modal_dialog_keypress (GtkWidget *w, GdkEventKey *e)
 {
-	if(e->keyval == GDK_Escape) {
+	if(e->keyval == GDK_KEY_Escape) {
 		gtk_widget_destroy (w);
 		return TRUE;
 	}
@@ -433,6 +559,16 @@ gu_delete_handler (GtkDialog *dialog,
 	return TRUE; /* Do not destroy */
 }
 
+static void
+recent_func_log_func (G_GNUC_UNUSED const gchar   *log_domain,
+		      G_GNUC_UNUSED GLogLevelFlags log_level,
+		      G_GNUC_UNUSED const gchar   *message,
+		      G_GNUC_UNUSED gpointer       user_data)
+{
+	/* We just want to suppress the warnings */
+	/* http://bugzilla.gnome.org/show_bug.cgi?id=664587 */
+}
+
 /**
  * go_gtk_file_sel_dialog :
  * @toplevel : #GtkWindow
@@ -447,6 +583,8 @@ go_gtk_file_sel_dialog (GtkWindow *toplevel, GtkWidget *w)
 {
 	gboolean result = FALSE;
 	gulong delete_handler;
+	guint log_handler;
+	const char *domain = "Gtk";
 
 	g_return_val_if_fail (GTK_IS_WINDOW (toplevel), FALSE);
 	g_return_val_if_fail (GTK_IS_FILE_CHOOSER (w), FALSE);
@@ -460,7 +598,14 @@ go_gtk_file_sel_dialog (GtkWindow *toplevel, GtkWidget *w)
 
 	gtk_widget_show (w);
 	gtk_grab_add (w);
+
+	/* Hack: the gtk file dialog warns when it can't access a new file, so shut it up.  */
+	/* http://bugzilla.gnome.org/show_bug.cgi?id=664587 */
+	/* This hack should be unneccessary in GTK 3.3.5 and later  */
+	log_handler = g_log_set_handler (domain, G_LOG_LEVEL_WARNING,
+					 recent_func_log_func, NULL);
 	gtk_main ();
+	g_log_remove_handler (domain, log_handler);
 
 	g_signal_handler_disconnect (w, delete_handler);
 
@@ -483,7 +628,7 @@ filter_images (const GtkFileFilterInfo *filter_info, gpointer data)
 		ext++;
 
 		if (!have_pixbufexts) {
-			GSList *l, *pixbuf_fmts = gdk_pixbuf_get_formats ();
+			GSList *pixbuf_fmts = gdk_pixbuf_get_formats ();
 
 			for (l = pixbuf_fmts; l != NULL; l = l->next) {
 				GdkPixbufFormat *fmt = l->data;
@@ -542,7 +687,7 @@ update_preview_cb (GtkFileChooser *chooser)
 		}
 
 		if (buf) {
-			GdkPixbuf *pixbuf = go_pixbuf_intelligent_scale (buf, PREVIEW_HSIZE, PREVIEW_VSIZE);
+			GdkPixbuf *pixbuf = go_gdk_pixbuf_intelligent_scale (buf, PREVIEW_HSIZE, PREVIEW_VSIZE);
 			gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
 			g_object_unref (pixbuf);
 			gtk_widget_show (image);
@@ -602,7 +747,7 @@ gui_image_chooser_new (gboolean is_save)
 
 	/* Preview */
 	{
-		GtkWidget *vbox = gtk_vbox_new (FALSE, 2);
+		GtkWidget *vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
 		GtkWidget *preview_image = gtk_image_new ();
 		GtkWidget *preview_label = gtk_label_new ("");
 
@@ -623,6 +768,12 @@ gui_image_chooser_new (gboolean is_save)
 
 char *
 go_gtk_select_image (GtkWindow *toplevel, char const *initial)
+{
+	return go_gtk_select_image_with_extra_widget (toplevel, initial, NULL);
+}
+
+char *
+go_gtk_select_image_with_extra_widget (GtkWindow *toplevel, char const *initial, GtkWidget *extra)
 {
 	char const *key = "go_gtk_select_image";
 	char *uri = NULL;
@@ -631,6 +782,8 @@ go_gtk_select_image (GtkWindow *toplevel, char const *initial)
 	g_return_val_if_fail (GTK_IS_WINDOW (toplevel), NULL);
 
 	fsel = gui_image_chooser_new (FALSE);
+	if (GTK_IS_WIDGET (extra))
+		gtk_file_chooser_set_extra_widget (fsel, extra);
 
 	if (!initial)
 		initial = g_object_get_data (G_OBJECT (toplevel), key);
@@ -717,8 +870,8 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 
 	g_object_set (G_OBJECT (fsel), "title", _("Save as"), NULL);
 
-	gui = go_gtk_builder_new ("go-image-save-dialog-extra.ui",
-			       GETTEXT_PACKAGE, NULL);
+	gui = go_gtk_builder_new_internal ("go-image-save-dialog-extra.ui",
+					   GETTEXT_PACKAGE, NULL);
 	if (gui != NULL) {
 		GtkWidget *widget;
 
@@ -730,7 +883,9 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 			for (l = supported_formats, i = 0; l != NULL; l = l->next, i++) {
 				format = GPOINTER_TO_UINT (l->data);
 				format_info = go_image_get_format_info (format);
-				gtk_combo_box_append_text (format_combo, _(format_info->desc));
+				if (format_info == NULL)
+					continue;
+				go_gtk_combo_box_append_text (format_combo, _(format_info->desc));
 				if (format == state->format)
 					gtk_combo_box_set_active (format_combo, i);
 			}
@@ -778,24 +933,26 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 	uri = gtk_file_chooser_get_uri (fsel);
 	if (format_combo) {
 		char *new_uri = NULL;
+		int index = gtk_combo_box_get_active (format_combo);
 
-		format = GPOINTER_TO_UINT (g_slist_nth_data
-			(supported_formats,
-			 gtk_combo_box_get_active (format_combo)));
-		format_info = go_image_get_format_info (format);
-		if (!go_url_check_extension (uri, format_info->ext, &new_uri) &&
-		    !go_gtk_query_yes_no (GTK_WINDOW (fsel), TRUE,
-		     _("The given file extension does not match the"
-		       " chosen file type. Do you want to use this name"
-		       " anyway?"))) {
-			g_free (new_uri);
-			g_free (uri);
-			uri = NULL;
-			goto loop;
-		} else {
+		if (index >= 0) {
+			format = GPOINTER_TO_UINT (g_slist_nth_data
+				(supported_formats, index));
+			format_info = go_image_get_format_info (format);
+			if (!go_url_check_extension (uri, format_info->ext, &new_uri) &&
+			    !go_gtk_query_yes_no (GTK_WINDOW (fsel), TRUE,
+			     _("The given file extension does not match the"
+			       " chosen file type. Do you want to use this name"
+			       " anyway?"))) {
+				g_free (new_uri);
+				g_free (uri);
+				uri = NULL;
+				goto loop;
+			}
 			g_free (uri);
 			uri = new_uri;
-		}
+		} else
+			format = GO_IMAGE_FORMAT_UNKNOWN;
 		*ret_format = format;
 	}
 	if (!go_gtk_url_is_writeable (GTK_WINDOW (fsel), uri, TRUE)) {
@@ -1137,69 +1294,6 @@ go_gtk_query_yes_no (GtkWindow *parent, gboolean default_answer,
 		go_gtk_dialog_run (GTK_DIALOG (dialog), parent);
 }
 
-#ifndef HAVE_GTK_WIDGET_SET_TOOLTIP_TEXT
-static GtkTooltips *
-get_tooltips (GObject *obj)
-{
-	GtkTooltips *tips = g_object_get_data (obj, "-go-tips");
-
-	if (!tips) {
-		tips = gtk_tooltips_new ();
-
-#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
-		g_object_ref_sink (tips);
-#else
-		g_object_ref (tips);
-		gtk_object_sink (GTK_OBJECT (tips));
-#endif
-		g_object_set_data_full (obj, "-go-tips", tips,
-					(GDestroyNotify)g_object_unref);
-	}
-
-	return tips;
-}
-#endif
-
-void
-go_widget_set_tooltip_text (GtkWidget *widget, char const *tip)
-{
-#ifdef HAVE_GTK_WIDGET_SET_TOOLTIP_TEXT
-	gtk_widget_set_tooltip_text (widget, tip);
-#else
-	GtkTooltips *tips = get_tooltips (G_OBJECT (widget));
-	gtk_tooltips_set_tip (tips, widget, tip, NULL);
-#endif
-}
-
-void
-go_tool_item_set_tooltip_text (GtkToolItem *item, char const *tip)
-{
-#ifdef HAVE_GTK_TOOL_ITEM_SET_TOOLTIP_TEXT
-	gtk_tool_item_set_tooltip_text (item, tip);
-#else
-	GtkTooltips *tips = get_tooltips (G_OBJECT (item));
-	gtk_tool_item_set_tooltip (item, tips, tip, NULL);
-#endif
-}
-
-
-#ifndef HAVE_GTK_DIALOG_GET_RESPONSE_FOR_WIDGET
-/* This is public from 2.8 onwards.   */
-static gint
-gtk_dialog_get_response_for_widget (GtkDialog *dialog, GtkWidget *widget)
-{
-	struct {
-		gint response_id;
-	} const *rd = g_object_get_data (G_OBJECT (widget),
-				   "gtk-dialog-response-data");
-	if (!rd)
-		return GTK_RESPONSE_NONE;
-	else
-		return rd->response_id;
-}
-#endif
-
-
 /**
  * go_dialog_guess_alternative_button_order:
  * @dialog : #GtkDialog
@@ -1312,7 +1406,7 @@ go_menu_position_below (GtkMenu  *menu,
 		sy += size.y;
 	}
 
-	gtk_widget_size_request (GTK_WIDGET (menu), &req);
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &req, NULL);
 
 	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
 		*x = sx;
@@ -1354,11 +1448,7 @@ go_menu_position_below (GtkMenu  *menu,
 GError *
 go_gtk_url_show (gchar const *url, GdkScreen *screen)
 {
-#if defined(HAVE_GTK_SHOW_URI)
 	GError *error = NULL;
 	gtk_show_uri (screen, url, GDK_CURRENT_TIME, &error);
 	return error;
-#else
-	return go_url_show (url);
-#endif
 }
