@@ -53,30 +53,6 @@ go_hash_keys (GHashTable *hash)
 	return accum;
 }
 
-static void
-cb_hash_collect_values (gpointer key, gpointer value, GSList **accum)
-{
-	*accum = g_slist_prepend (*accum, value);
-}
-
-/**
- * go_hash_values :
- * @hash : #GHashTable
- *
- * Collects an unordered list of the values in @hash.
- *
- * Returns: a list which the caller needs to free.
- * 	The content has not additional references added
- **/
-GSList *
-go_hash_values (GHashTable *hash)
-{
-	GSList *accum = NULL;
-	g_hash_table_foreach (hash,
-		(GHFunc )cb_hash_collect_values, &accum);
-	return accum;
-}
-
 /***************************************************************************/
 void
 go_ptr_array_insert (GPtrArray *array, gpointer value, int index)
@@ -97,7 +73,7 @@ go_ptr_array_insert (GPtrArray *array, gpointer value, int index)
 
 /**
  * go_slist_create:
- * @item1: itionally %NULL
+ * @item1: optionally %NULL
  * @Varargs : %NULL terminated list of additional items
  *
  * Creates a GList from NULL-terminated list of arguments.
@@ -138,24 +114,6 @@ go_slist_map (GSList const *list, GOMapFunc map_func)
 	return g_slist_reverse (list_copy);
 }
 
-/**
- * go_slist_free_custom:
- * @list: list of some items
- * @free_func: function freeing list item
- *
- * Clears a list, calling @free_func for each list item.
- **/
-void
-go_slist_free_custom (GSList *list, GFreeFunc free_func)
-{
-	GSList *l;
-
-	for (l = list; l != NULL; l = l->next) {
-		free_func (l->data);
-	}
-	g_slist_free (list);
-}
-
 gint
 go_list_index_custom (GList *list, gpointer data, GCompareFunc cmp_func)
 {
@@ -171,25 +129,6 @@ go_list_index_custom (GList *list, gpointer data, GCompareFunc cmp_func)
 	return -1;
 }
 
-/**
- * go_list_free_custom:
- * @list: list of some items
- * @free_func: function freeing list item
- *
- * Clears a list, calling @free_func for each list item.
- *
- */
-void
-go_list_free_custom (GList *list, GFreeFunc free_func)
-{
-	GList *l;
-
-	for (l = list; l != NULL; l = l->next) {
-		free_func (l->data);
-	}
-	g_list_free (list);
-}
-
 /**
  * go_strsplit_to_slist:
  * @str: String to split
@@ -695,7 +634,7 @@ go_str_compare (void const *x, void const *y)
 
 const char *
 go_guess_encoding (const char *raw, size_t len, const char *user_guess,
-		   char **utf8_str)
+		   GString **utf8_str, guint *truncated)
 {
 	int try;
 	gboolean debug = FALSE;
@@ -706,6 +645,8 @@ go_guess_encoding (const char *raw, size_t len, const char *user_guess,
 		char const *guess = NULL;
 		GError *error = NULL;
 		char *utf8_data;
+		gsize bytes_written = 0;
+		gsize bytes_read = 0;
 
 		switch (try) {
 		case 1: guess = user_guess; break;
@@ -743,20 +684,24 @@ go_guess_encoding (const char *raw, size_t len, const char *user_guess,
 			g_print ("Trying %s as encoding.\n", guess);
 
 		utf8_data = g_convert (raw, len, "UTF-8", guess,
-				       NULL, NULL, &error);
+				       &bytes_read, &bytes_written, &error);
 		if (!error) {
 			/*
-			 * We can actually fail this test when gues is UTF-8,
+			 * We can actually fail this test when guess is UTF-8,
 			 * see #401588.
 			 */
-			if (!g_utf8_validate (utf8_data, -1, NULL))
+			if (!g_utf8_validate (utf8_data, -1, NULL)) {
+				g_free (utf8_data);
 				continue;
+			}
 			if (debug)
 				g_print ("Guessed %s as encoding.\n", guess);
 			if (utf8_str)
-				*utf8_str = utf8_data;
-			else
-				g_free (utf8_data);
+				*utf8_str = g_string_new_len
+					(utf8_data, bytes_written);
+			g_free (utf8_data);
+			if (truncated)
+				*truncated = len - bytes_read;
 			return guess;
 		}
 
@@ -764,6 +709,8 @@ go_guess_encoding (const char *raw, size_t len, const char *user_guess,
 	}
 }
 
+static char *go_real_name = NULL;
+
 /**
  * go_get_real_name :
  *
@@ -773,20 +720,21 @@ go_guess_encoding (const char *raw, size_t len, const char *user_guess,
 char const *
 go_get_real_name (void)
 {
-	/* We will leak this.  */
-	static char *go_real_name = NULL;
-
 	if (go_real_name == NULL) {
 		char const *name = getenv ("NAME");
 		if (name == NULL)
 			name = g_get_real_name ();
 		if (name == NULL)
 			name = g_get_user_name ();
-		if (name != NULL)
+		if (name != NULL) {
+			GString *converted_name = NULL;
 			(void) go_guess_encoding (name, strlen (name),
-				NULL, &go_real_name);
-		else
-			go_real_name = (char *)"unknown";
+						  NULL, &converted_name, NULL);
+			if (converted_name)
+				go_real_name = g_string_free (converted_name, FALSE);
+		}
+		if (go_real_name == NULL)
+			go_real_name = g_strdup ("unknown");
 	}
 	return go_real_name;
 }
@@ -1080,3 +1028,10 @@ syntax:
 	res = TRUE;
 	goto done;
 }
+
+void
+_go_glib_extras_shutdown (void)
+{
+	g_free (go_real_name);
+	go_real_name = NULL;
+}
