@@ -20,14 +20,12 @@
  */
 
 #include <goffice/goffice-config.h>
-#include <goffice/utils/go-color.h>
-#include <goffice/utils/go-image.h>
-#include <goffice/utils/go-cairo.h>
-#include <glib/gi18n-lib.h>
+#include <goffice/goffice.h>
 #include <string.h>
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
+#include <librsvg/rsvg.h>
 
 static GOImageFormatInfo *pixbuf_image_format_infos = NULL;
 static GHashTable *pixbuf_mimes = NULL;
@@ -59,9 +57,7 @@ go_mime_to_image_format (char const *mime_type)
 		"image/x-emf", "emf",
 		"application/pdf", "pdf",
 		"application/postscript", "ps",
-#ifdef HAVE_CAIRO_PS_SURFACE_SET_EPS
 		"image/x-eps", "eps",
-#endif
 	};
 
 	for (i = 0; i < G_N_ELEMENTS (exceptions); i += 2)
@@ -85,18 +81,14 @@ go_image_format_to_mime (char const *format)
 {
 	char *ret = NULL;
  	guint i;
-#ifdef GOFFICE_WITH_GTK
 	GSList *ptr, *pixbuf_fmts;
-#endif
 	static const char* const formats[] = {
 		"svg", "image/svg,image/svg+xml",
 		"wmf", "image/x-wmf",
 		"emf", "image/x-emf",
 		"pdf", "application/pdf",
 		"ps", "application/postscript",
-#ifdef HAVE_CAIRO_PS_SURFACE_SET_EPS
 		"eps", "image/x-eps",
-#endif
 	};
 
 	if (format == NULL)
@@ -106,7 +98,6 @@ go_image_format_to_mime (char const *format)
 		if (strcmp (format, formats[i]) == 0)
 			return g_strdup (formats[i + 1]);
 
-#ifdef GOFFICE_WITH_GTK
 	/* Not a format we have special knowledge about - ask gdk-pixbuf */
 	pixbuf_fmts = gdk_pixbuf_get_formats ();
 	for (ptr = pixbuf_fmts; ptr != NULL; ptr = ptr->next) {
@@ -123,7 +114,6 @@ go_image_format_to_mime (char const *format)
 		}
 	}
 	g_slist_free (pixbuf_fmts);
-#endif
 
 	return ret;
 }
@@ -143,19 +133,13 @@ static GOImageFormatInfo const image_format_infos[GO_IMAGE_FORMAT_UNKNOWN] = {
 	 (char *) "emf", FALSE, FALSE, TRUE},
 	{GO_IMAGE_FORMAT_WMF, (char *) "wmf",  (char *) N_("WMF (windows metafile)"),
 	 (char *) "wmf", FALSE, FALSE, TRUE},
-#ifdef HAVE_CAIRO_PS_SURFACE_SET_EPS
 	{GO_IMAGE_FORMAT_EPS,  (char *) "eps",   (char *) N_("EPS (encapsulated postscript)"),
 	 (char *) "eps",  FALSE, TRUE, TRUE},
-#else
-	{GO_IMAGE_FORMAT_EPS,  (char *) "",   (char *) "",
-	 (char *) "",  FALSE, FALSE, FALSE},
-#endif
 };
 
 static void
 go_image_build_pixbuf_format_infos (void)
 {
-#ifdef GOFFICE_WITH_GTK
 	GdkPixbufFormat *fmt;
 	GSList *l, *pixbuf_fmts;
 	GOImageFormatInfo *format_info;
@@ -202,7 +186,6 @@ go_image_build_pixbuf_format_infos (void)
 	}
 
 	g_slist_free (pixbuf_fmts);
-#endif /* GOFFICE_WITH_GTK */
 	pixbuf_format_done = TRUE;
 }
 
@@ -210,7 +193,7 @@ go_image_build_pixbuf_format_infos (void)
  * go_image_get_format_info:
  * @format: a #GOImageFormat
  *
- * Retrieves infromation associated to @format.
+ * Retrieves information associated to @format.
  *
  * returns: a #GOImageFormatInfo struct.
  **/
@@ -218,11 +201,12 @@ go_image_build_pixbuf_format_infos (void)
 GOImageFormatInfo const *
 go_image_get_format_info (GOImageFormat format)
 {
+	if (format == GO_IMAGE_FORMAT_UNKNOWN)
+		return NULL;
 	if (format > GO_IMAGE_FORMAT_UNKNOWN)
 		go_image_build_pixbuf_format_infos ();
 
 	g_return_val_if_fail (format >= 0 &&
-			      format != GO_IMAGE_FORMAT_UNKNOWN &&
 			      format <= GO_IMAGE_FORMAT_UNKNOWN + pixbuf_format_nbr, NULL);
 	if (format < GO_IMAGE_FORMAT_UNKNOWN)
 		return &image_format_infos[format];
@@ -242,6 +226,9 @@ go_image_get_format_from_name (char const *name)
 {
 	unsigned i;
 
+	if (name == NULL)
+		return GO_IMAGE_FORMAT_UNKNOWN;
+
 	go_image_build_pixbuf_format_infos ();
 
 	for (i = 0; i < GO_IMAGE_FORMAT_UNKNOWN; i++) {
@@ -293,63 +280,12 @@ go_image_get_formats_with_pixbuf_saver (void)
 
 static GObjectClass *parent_klass;
 
-struct _GOImage {
-	GObject parent;
-	guint8 *data;
-	guint width, height, rowstride;
-	gboolean target_cairo;
-	cairo_t *cairo;
-#ifdef GOFFICE_WITH_GTK
-	GdkPixbuf *pixbuf, *thumbnail;
-#else
-	void *pixbuf;
-#endif
-	char *name;
-};
-
 enum {
 	IMAGE_PROP_0,
 	IMAGE_PROP_WIDTH,
-	IMAGE_PROP_HEIGHT,
-#ifdef GOFFICE_WITH_GTK
-	IMAGE_PROP_PIXBUF,
-#endif
+	IMAGE_PROP_HEIGHT
 };
 
-static void
-pixbuf_to_cairo (GOImage *image)
-{
-#ifdef GOFFICE_WITH_GTK
-	unsigned char *src, *dst;
-
-	g_return_if_fail (GO_IS_IMAGE (image) && image->data && image->pixbuf);
-
-	src = gdk_pixbuf_get_pixels (image->pixbuf);
-	dst = image->data;
-
-	g_return_if_fail (gdk_pixbuf_get_rowstride (image->pixbuf) == (int) image->rowstride);
-
-	go_cairo_convert_data_from_pixbuf (dst, src, image->width, image->height, image->rowstride);
-#endif
-}
-
-#ifdef GOFFICE_WITH_GTK
-static void
-cairo_to_pixbuf (GOImage *image)
-{
-	unsigned char *src, *dst;
-
-	g_return_if_fail (GO_IS_IMAGE (image) && image->data && image->pixbuf);
-
-	dst = gdk_pixbuf_get_pixels (image->pixbuf);
-	src = image->data;
-
-	g_return_if_fail (gdk_pixbuf_get_rowstride (image->pixbuf) == (int) image->rowstride);
-
-	go_cairo_convert_data_to_pixbuf (dst, src, image->width, image->height, image->rowstride);
-}
-#endif
-
 static void
 go_image_set_property (GObject *obj, guint param_id,
 		       GValue const *value, GParamSpec *pspec)
@@ -373,46 +309,14 @@ go_image_set_property (GObject *obj, guint param_id,
 			size_changed = TRUE;
 		}
 		break;
-#ifdef GOFFICE_WITH_GTK
-	case IMAGE_PROP_PIXBUF: {
-		GdkPixbuf *pixbuf = GDK_PIXBUF (g_value_get_object (value));
-		if (!GDK_IS_PIXBUF (pixbuf))
-			break;
-		if (!gdk_pixbuf_get_has_alpha (pixbuf))
-			pixbuf = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
-		else
-			g_object_ref (pixbuf);
-		if (image->pixbuf)
-			g_object_unref (image->pixbuf);
-		image->pixbuf = pixbuf;
-		g_free (image->data);
-		image->data = NULL;
-		image->width = gdk_pixbuf_get_width (pixbuf);
-		image->height = gdk_pixbuf_get_height (pixbuf);
-		image->rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-		image->target_cairo = FALSE;
-		if (image->thumbnail) {
-			g_object_unref (image->thumbnail);
-			image->thumbnail = NULL;
-		}
-	}
-		break;
-#endif
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		return; /* NOTE : RETURN */
 	}
 
 	if (size_changed) {
-		if (image->pixbuf) {
-			g_object_unref (image->pixbuf);
-			image->pixbuf = NULL;
-		}
 		g_free (image->data);
-		/* GOImage only supports pixbuf with alpha values at the moment */
-		image->rowstride = image->width * 4;
-		image->data = g_new0 (guint8, image->height * image->rowstride);
-		image->target_cairo = TRUE;
+		image->data = NULL;
 	}
 }
 
@@ -429,15 +333,6 @@ go_image_get_property (GObject *obj, guint param_id,
 	case IMAGE_PROP_HEIGHT:
 		g_value_set_uint (value, image->height);
 		break;
-#ifdef GOFFICE_WITH_GTK
-	case IMAGE_PROP_PIXBUF:
-		if (image->target_cairo && image->pixbuf) {
-			cairo_to_pixbuf (image);
-			image->target_cairo = FALSE;
-		}
-		g_value_set_object (value, image->pixbuf);
-		break;
-#endif
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		return; /* NOTE : RETURN */
@@ -449,20 +344,16 @@ go_image_finalize (GObject *obj)
 {
 	GOImage *image = GO_IMAGE (obj);
 	g_free (image->data);
-#ifdef GOFFICE_WITH_GTK
-	if (image->pixbuf)
-		g_object_unref (image->pixbuf);
 	if (image->thumbnail)
 		g_object_unref (image->thumbnail);
-#endif
+	if (image->pixbuf)
+		g_object_unref (image->pixbuf);
 	g_free (image->name);
 	(parent_klass->finalize) (obj);
 }
 
-typedef GObjectClass GOImageClass;
-
 static void
-go_image_class_init (GOImageClass *klass)
+go_image_class_init (GObjectClass *klass)
 {
 	klass->finalize = go_image_finalize;
 	klass->set_property = go_image_set_property;
@@ -476,167 +367,157 @@ go_image_class_init (GOImageClass *klass)
 					 g_param_spec_uint ("height", _("Height"),
 							    _("Image height in pixels"),
 							    0, G_MAXUINT16, 0, G_PARAM_READWRITE));
-#ifdef GOFFICE_WITH_GTK
-	g_object_class_install_property (klass, IMAGE_PROP_PIXBUF,
-					 g_param_spec_object ("pixbuf", _("Pixbuf"),
-							      _("GdkPixbuf object from which the GOImage is built"),
-							      GDK_TYPE_PIXBUF, G_PARAM_READWRITE));
-#endif
 }
 
-GSF_CLASS (GOImage, go_image,
-	   go_image_class_init, NULL,
-	   G_TYPE_OBJECT)
+GSF_CLASS_ABSTRACT (GOImage, go_image,
+		go_image_class_init, NULL,
+		G_TYPE_OBJECT)
 
-cairo_t *
-go_image_get_cairo (GOImage *image)
+void
+go_image_draw (GOImage *image, cairo_t *cr)
 {
-	cairo_surface_t *surface ;
-	cairo_t *cairo;
-
-	g_return_val_if_fail (GO_IS_IMAGE (image), NULL);
-	if (image->data == NULL && image->pixbuf == NULL)
-		return NULL;
-	if (image->data == NULL) {
-		/* image built from a pixbuf */
-		image->data = g_new0 (guint8, image->height * image->rowstride);
-	}
-	if (!image->target_cairo) {
-		pixbuf_to_cairo (image);
-		image->target_cairo = TRUE;
-	}
-	surface = cairo_image_surface_create_for_data (
-						       image->data,
-						       CAIRO_FORMAT_ARGB32,
-						       image->width, image->height,
-						       image->rowstride);
-	cairo = cairo_create (surface);
-	cairo_surface_destroy (surface);
-	image->target_cairo = TRUE;
-	return cairo;
+	g_return_if_fail (GO_IS_IMAGE (image));
+	((GOImageClass *) G_OBJECT_GET_CLASS (image))->draw (image, cr);
 }
 
-static void
-cb_surface_destroyed (void *data)
-{
-	GOImage *image = GO_IMAGE (data);
-	/* We no longer need image->data.  */
-	g_object_unref (image);
-}
+#define GO_THUMBNAIL_SIZE 64
 
-/**
- * go_image_create_cairo_pattern:
- * @image: a GOImage.
- *
- * returns: a cairo_pattern usable for cairo_set_source.
- *
- * Note: this function has lifespan issues.  The resulting pattern in only
- * valid until (a) a pixbuf is set for the, or (b) a pixbuf is _read_ from
- * the image.  In either of these cases, the pattern must have been
- * destroyed beforehand.  In particular, if the pattern has been attached
- * to a surface, that surface must either be finished itself, or have had
- * a new pattern attached.  See #632439.
- */
-cairo_pattern_t *
-go_image_create_cairo_pattern (GOImage *image)
-{
-	cairo_surface_t *surface ;
-	cairo_pattern_t *pat;
-	static const cairo_user_data_key_t key;
-
-	g_return_val_if_fail (GO_IS_IMAGE (image), NULL);
-	if (image->data == NULL && image->pixbuf == NULL)
-		return NULL;
-	if (image->data == NULL) {
-		/* image built from a pixbuf */
-		image->data = g_new0 (guint8, image->height * image->rowstride);
-	}
-	if (!image->target_cairo) {
-		pixbuf_to_cairo (image);
-		image->target_cairo = TRUE;
-	}
-	surface = cairo_image_surface_create_for_data
-		(image->data,
-		 CAIRO_FORMAT_ARGB32,
-		 image->width, image->height,
-		 image->rowstride);
-	g_object_ref (image);
-	cairo_surface_set_user_data (surface, &key,
-				     image, cb_surface_destroyed);
-	pat = cairo_pattern_create_for_surface (surface);
-	cairo_surface_destroy (surface);
-	return pat;
-}
-
-#ifdef GOFFICE_WITH_GTK
-GOImage *
-go_image_new_from_pixbuf (GdkPixbuf *pixbuf)
-{
-	return g_object_new (GO_TYPE_IMAGE, "pixbuf", pixbuf, NULL);
-}
-
-GdkPixbuf *
-go_image_get_pixbuf (GOImage *image)
-{
-	g_return_val_if_fail (image != NULL, NULL);
-	if (!image->pixbuf) {
-		if (image->width == 0 || image->height == 0 || image->data == NULL)
-			return NULL;
-		image->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-								image->width, image->height);
-	}
-	if (image->target_cairo) {
-		cairo_to_pixbuf (image);
-		image->target_cairo = FALSE;
-	}
-	return image->pixbuf;
-}
-
-#define THUMBNAIL_SIZE 64
-GdkPixbuf *
+GdkPixbuf const *
 go_image_get_thumbnail (GOImage *image)
 {
 	g_return_val_if_fail (image != NULL, NULL);
-	if (!image->pixbuf)
-		go_image_get_pixbuf (image);
-	if (!image->pixbuf)
-		return NULL;
-	if (!image->thumbnail) {
-		int w, h;
-		if (image->width <= THUMBNAIL_SIZE && image->height <= THUMBNAIL_SIZE)
-			return image->pixbuf;
-		if (image->width >= image->height) {
-			w = THUMBNAIL_SIZE;
-			h = THUMBNAIL_SIZE * image->height / image->width;
-		} else {
-			h = THUMBNAIL_SIZE;
-			w = THUMBNAIL_SIZE * image->width / image->height;
-		}
-		image->thumbnail = gdk_pixbuf_scale_simple (image->pixbuf, w, h, GDK_INTERP_HYPER);
-	}
+	if (image->thumbnail == NULL)
+		image->thumbnail = go_image_get_scaled_pixbuf (image, GO_THUMBNAIL_SIZE, GO_THUMBNAIL_SIZE);
 	return image->thumbnail;
 }
-#endif
+GdkPixbuf *
+go_image_get_pixbuf (GOImage *image)
+{
+	if (image->pixbuf == NULL)
+		image->pixbuf = ((GOImageClass *) G_OBJECT_GET_CLASS (image))->get_pixbuf (image);
+	return g_object_ref (image->pixbuf);
+}
+
+GdkPixbuf *
+go_image_get_scaled_pixbuf (GOImage *image, int width, int height)
+{
+	if (image->width > width || image->height > height) {
+		if (image->width * height > image->height * width) {
+			height = width * image->height / image->width;
+		} else {
+			width = height * image->width / image->height;
+		}
+	} else
+		return ((GOImageClass *) G_OBJECT_GET_CLASS (image))->get_pixbuf (image);
+	return ((GOImageClass *) G_OBJECT_GET_CLASS (image))->get_scaled_pixbuf (image, width, height);
+}
 
 GOImage *
-go_image_new_from_file (const char *filename, GError **error)
+go_image_new_from_file (char const *filename, GError **error)
 {
-#ifdef GOFFICE_WITH_GTK
-	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (filename, error);
-	if (pixbuf) {
-		GOImage *image = g_object_new (GO_TYPE_IMAGE,
-					       "pixbuf", pixbuf,
-					       NULL);
-		g_object_unref (pixbuf);
-		image->target_cairo = FALSE;
-		return image;
+	char *mime, *name;
+	GOImageFormat format;
+
+	if (!filename) {
+		return NULL;
 	}
-#else
-	g_warning ("go_image_new_from_file not implemented!");
+	mime = go_get_mime_type (filename);
+	if (!mime) {
+		return NULL;
+	}
+	name = go_mime_to_image_format (mime);
+	g_free (mime);
+	if (!name) {
+		return NULL;
+	}
+	format = go_image_get_format_from_name (name);
+	g_free (name);
+	switch (format) {
+	case GO_IMAGE_FORMAT_SVG:
+		return go_svg_new_from_file (filename, error);
+	case GO_IMAGE_FORMAT_EMF:
+	case GO_IMAGE_FORMAT_WMF:
+		return go_emf_new_from_file (filename, error);
+	case GO_IMAGE_FORMAT_EPS:
+#ifdef GOFFICE_WITH_EPS
+		return go_spectre_new_from_file (filename, error);
 #endif
+	case GO_IMAGE_FORMAT_PS: /* we might support ps with libspectre */
+	case GO_IMAGE_FORMAT_PDF:
+	case GO_IMAGE_FORMAT_UNKNOWN:
+		break;
+	default: {
+		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (filename, error);
+		if (pixbuf) {
+			GOImage *image = go_pixbuf_new_from_pixbuf (pixbuf);
+			g_object_unref (pixbuf);
+			return image;
+		}
+	}
+	}
 	return NULL;
 }
 
+GOImage	*
+go_image_new_from_data (char const *type, guint8 const *data, gsize length, char **format, GError **error)
+{
+	char *real_type = NULL;
+	GOImage *image = NULL;
+	if (type == NULL || *type == 0) {
+		char *mime_type = go_get_mime_type_for_data (data, length);
+		real_type = go_mime_to_image_format (mime_type);
+		g_free (mime_type);
+		type = real_type;
+	}
+	if (type == NULL) {
+		g_warning ("unrecognized image format");
+		return NULL;
+	}
+	if (!strcmp (type, "svg")) {
+		image = go_svg_new_from_data (data, length, error);
+	} else if (!strcmp (type, "emf") || !strcmp (type, "wmf")) {
+		image = go_emf_new_from_data (data, length, error);
+#ifdef GOFFICE_WITH_EPS
+	} else if (!strcmp (type, "eps")) {
+		image = go_spectre_new_from_data (data, length, error);
+#endif
+	} else {
+		GdkPixbufLoader *loader = gdk_pixbuf_loader_new_with_type (type, error);
+		if (loader) {
+			if (gdk_pixbuf_loader_write (loader, data, length, error))
+				image = go_pixbuf_new_from_pixbuf (gdk_pixbuf_loader_get_pixbuf (loader));
+			gdk_pixbuf_loader_close (loader, error);
+			g_object_unref (loader);
+		}
+	}
+	if (format)
+		*format = g_strdup (type);
+	g_free (real_type);
+	return image;
+}
+
+GOImage *
+go_image_new_for_format (char const *format)
+{
+	GType gtype = go_image_type_for_format (format);
+	return (gtype > 0)? g_object_new (gtype, NULL): NULL;
+}
+
+GType
+go_image_type_for_format (char const *format)
+{
+	g_return_val_if_fail (format && *format, 0);
+	if (!strcmp (format, "svg"))
+		return GO_TYPE_SVG;
+	if (!strcmp (format, "emf") || !strcmp (format, "wmf"))
+		return GO_TYPE_EMF;
+	 if (!strcmp (format, "eps"))
+		return GO_TYPE_SPECTRE;
+	if (go_image_get_format_from_name (format) != GO_IMAGE_FORMAT_UNKNOWN)
+		return GO_TYPE_PIXBUF;
+	return 0;
+}
+
 guint8 *
 go_image_get_pixels (GOImage *image)
 {
@@ -644,34 +525,6 @@ go_image_get_pixels (GOImage *image)
 	return image->data;
 }
 
-int
-go_image_get_rowstride (GOImage *image)
-{
-	g_return_val_if_fail (image, 0);
-	return image->rowstride;
-}
-
-void
-go_image_fill (GOImage *image, GOColor color)
-{
-	guint32 val;
-	guint8 *dst;
-	unsigned i, j;
-	g_return_if_fail (image);
-
-	dst = image->data;
-	if (image->target_cairo)
-		val = (GO_COLOR_UINT_R (color) << 8) + (GO_COLOR_UINT_G (color) << 16)
-			+ (GO_COLOR_UINT_B (color) << 24) + GO_COLOR_UINT_A (color);
-	else
-		val = color;
-	for (i = 0; i < image->height; i++) {
-		for (j = 0; j < image->width; j++)
-			*((guint32*) dst) = val;
-		dst += image->rowstride - image->width * 4;
-	}
-}
-
 void
 go_image_set_name (GOImage *image, char const *name)
 {
@@ -680,47 +533,22 @@ go_image_set_name (GOImage *image, char const *name)
 }
 
 char const *
-go_image_get_name (GOImage *image)
+go_image_get_name (GOImage const *image)
 {
 	g_return_val_if_fail (GO_IS_IMAGE (image), NULL);
 	return image->name;
 }
 
 gboolean
-go_image_same_pixbuf (GOImage *first, GOImage *second)
+go_image_differ (GOImage *first, GOImage *second)
 {
-#ifdef GOFFICE_WITH_GTK
-	void *pixels1, *pixels2;
-	int size;
 	g_return_val_if_fail (GO_IS_IMAGE (first), FALSE);
 	g_return_val_if_fail (GO_IS_IMAGE (second), FALSE);
-	if (!first->pixbuf)
-		go_image_get_pixbuf (first);
-	if (!second->pixbuf)
-		go_image_get_pixbuf (second);
-	if (!first->pixbuf || !second->pixbuf)
-		return FALSE;
-	if (gdk_pixbuf_get_n_channels (first->pixbuf) != gdk_pixbuf_get_n_channels (second->pixbuf))
-		return FALSE;
-	if (gdk_pixbuf_get_colorspace (first->pixbuf) != gdk_pixbuf_get_colorspace (second->pixbuf))
-		return FALSE;
-	if (gdk_pixbuf_get_bits_per_sample (first->pixbuf) != gdk_pixbuf_get_bits_per_sample (second->pixbuf))
-		return FALSE;
-	if (gdk_pixbuf_get_has_alpha (first->pixbuf) != gdk_pixbuf_get_has_alpha (second->pixbuf))
-		return FALSE;
-	if (gdk_pixbuf_get_width (first->pixbuf) != gdk_pixbuf_get_width (second->pixbuf))
-		return FALSE;
-	if (gdk_pixbuf_get_height (first->pixbuf) != gdk_pixbuf_get_height (second->pixbuf))
-		return FALSE;
-	if (gdk_pixbuf_get_rowstride (first->pixbuf) != gdk_pixbuf_get_rowstride (second->pixbuf))
-		return FALSE;
-	pixels1 = gdk_pixbuf_get_pixels (first->pixbuf);
-	pixels2 = gdk_pixbuf_get_pixels (second->pixbuf);
-	size = gdk_pixbuf_get_rowstride (first->pixbuf) * gdk_pixbuf_get_height (first->pixbuf);
-	return !memcmp (pixels1, pixels2, size);
-#else
-	return FALSE;
-#endif
+	if (G_OBJECT_TYPE (first) != G_OBJECT_TYPE (second))
+		return TRUE;
+	if (first->width != second->width || first->height != second->height)
+		return TRUE;
+	return ((GOImageClass *) G_OBJECT_GET_CLASS (first))->differ (first, second);
 }
 
 void
@@ -729,11 +557,10 @@ go_image_save (GOImage *image, GsfXMLOut *output)
 	g_return_if_fail (GO_IS_IMAGE (image) && image->name);
 	gsf_xml_out_start_element (output, "GOImage");
 	gsf_xml_out_add_cstr (output, "name", image->name);
+	gsf_xml_out_add_cstr (output, "type", G_OBJECT_TYPE_NAME (image));
 	gsf_xml_out_add_int (output, "width", image->width);
 	gsf_xml_out_add_int (output, "height", image->height);
-	gsf_xml_out_add_int (output, "rowstride", image->rowstride);
-	gsf_xml_out_add_base64 (output, NULL,
-			go_image_get_pixels (image), image->height * image->rowstride);
+	((GOImageClass *) G_OBJECT_GET_CLASS (image))->save (image, output);
 	gsf_xml_out_end_element (output);
 }
 
@@ -747,16 +574,51 @@ go_image_load_attrs (GOImage *image, GsfXMLIn *xin, xmlChar const **attrs)
 			image->width = strtol (attr[1], NULL, 10);
 		else if (0 == strcmp (attr[0], "height"))
 			image->height= strtol (attr[1], NULL, 10);
-		else if (0 == strcmp (attr[0], "rowstride"))
-			image->rowstride = strtol (attr[1], NULL, 10);
+		else
+			((GOImageClass *) G_OBJECT_GET_CLASS (image))->load_attr (image, attr[0], attr[1]);
 }
 
 void
 go_image_load_data (GOImage *image, GsfXMLIn *xin)
 {
-	int length;
-	length = gsf_base64_decode_simple (xin->content->str, strlen(xin->content->str));
-	image->data = g_memdup (xin->content->str, length);
-	image->target_cairo = TRUE;
+	((GOImageClass *) G_OBJECT_GET_CLASS (image))->load_data (image, xin);
 }
 
+void
+_go_image_changed (GOImage *image, double width, double height)
+{
+	image->width = width;
+	image->height = height;
+	if (image->thumbnail) {
+		g_object_unref (image->thumbnail);
+		image->thumbnail = NULL;
+	}
+}
+
+double
+go_image_get_width (GOImage const *image)
+{
+	return image->width;
+}
+
+double
+go_image_get_height (GOImage const *image)
+{
+	return image->height;
+}
+
+static double _go_image_dpi_x = 96.,  _go_image_dpi_y = 96.;
+
+void
+go_image_set_default_dpi (double dpi_x, double dpi_y)
+{
+	_go_image_dpi_x = dpi_x;
+	_go_image_dpi_y = dpi_y;
+}
+
+void
+go_image_get_default_dpi (double *dpi_x, double *dpi_y)
+{
+	*dpi_x = _go_image_dpi_x;
+	*dpi_y = _go_image_dpi_y;
+}
