@@ -67,7 +67,6 @@ typedef struct {
 		GtkWidget *foreground_box;
 		GtkWidget *foreground_label;
 		GtkWidget *notebook;
-		GtkWidget *extension_box;
 		struct {
 			GtkWidget *selector;
 			GtkWidget *box;
@@ -115,6 +114,7 @@ create_go_combo_color (StylePrefState *state,
 	GtkWidget *w;
 
 	w = go_color_selector_new (initial_color, automatic_color, group);
+	gtk_widget_set_halign (w, GTK_ALIGN_START);
 	gtk_label_set_mnemonic_widget (
 		GTK_LABEL (gtk_builder_get_object (gui, label_name)), w);
 	g_signal_connect (G_OBJECT (w), "activate", G_CALLBACK (func), state);
@@ -137,14 +137,18 @@ go_style_set_image_preview (GOImage *pix, StylePrefState *state)
 			g_object_ref (state->fill.image.image);
 	}
 
-	if (pix == NULL)
+	if (pix == NULL) {
+		gtk_widget_hide (go_gtk_builder_get_widget (state->gui, "fill_image_sample"));
+		gtk_label_set_text (GTK_LABEL (go_gtk_builder_get_widget (state->gui, "image-size-label")), _("No image!"));
 		return;
+	}
 
 	w = go_gtk_builder_get_widget (state->gui, "fill_image_sample");
 
-	scaled = go_pixbuf_intelligent_scale (go_image_get_pixbuf (pix), HSCALE, VSCALE);
+	scaled = go_image_get_scaled_pixbuf (pix, HSCALE, VSCALE);
 	gtk_image_set_from_pixbuf (GTK_IMAGE (w), scaled);
 	g_object_unref (scaled);
+	gtk_widget_show (w);
 
 	w = go_gtk_builder_get_widget (state->gui, "image-size-label");
 	g_object_get (pix, "width", &width, "height", &height, NULL);
@@ -194,22 +198,21 @@ outline_init (StylePrefState *state, gboolean enable, GOEditor *editor)
 {
 	GOStyle *style = state->style;
 	GOStyle *default_style = state->default_style;
-	GtkWidget *w, *table;
+	GtkWidget *w, *grid;
 
-	w = go_gtk_builder_get_widget (state->gui, "outline_box");
+	grid = go_gtk_builder_get_widget (state->gui, "outline-grid");
 	if (!enable) {
+		gtk_widget_hide (grid);
+		w = go_gtk_builder_get_widget (state->gui, "outline-label");
 		gtk_widget_hide (w);
 		return;
 	}
 
-	go_editor_register_widget (editor, w);
-
-	table = go_gtk_builder_get_widget (state->gui, "outline_table");
-
 	/* DashType */
 	w = go_line_dash_selector_new (style->line.dash_type,
 				       default_style->line.dash_type);
-	gtk_table_attach (GTK_TABLE (table), w, 1, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_widget_set_halign (w, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), w, 1, 0, 2, 1);
 	g_signal_connect (G_OBJECT (w), "activate",
 			  G_CALLBACK (cb_outline_dash_type_changed), state);
 	/* Size */
@@ -224,8 +227,8 @@ outline_init (StylePrefState *state, gboolean enable, GOEditor *editor)
 		state->gui,
 		"outline_color", "outline_color_label",
 		G_CALLBACK (cb_outline_color_changed));
-	gtk_table_attach (GTK_TABLE (table), w, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-	gtk_widget_show_all (table);
+	gtk_grid_attach (GTK_GRID (grid), w, 1, 1, 1, 1);
+	gtk_widget_show_all (grid);
 }
 
 
@@ -268,22 +271,23 @@ line_init (StylePrefState *state, gboolean enable, GOEditor *editor)
 {
 	GOStyle *style = state->style;
 	GOStyle *default_style = state->default_style;
-	GtkWidget *w, *table;
+	GtkWidget *w, *grid;
 
-	w = go_gtk_builder_get_widget (state->gui, "line_box");
+	grid = go_gtk_builder_get_widget (state->gui, "line-grid");
 	if (!enable) {
+		gtk_widget_hide (grid);
+		w = go_gtk_builder_get_widget (state->gui, "line-label");
 		gtk_widget_hide (w);
 		return;
 	}
 
-	go_editor_register_widget (editor, w);
-
-	table = go_gtk_builder_get_widget (state->gui, "line_table");
+	go_editor_register_widget (editor, grid);
 
 	/* DashType */
 	w = go_line_dash_selector_new (style->line.dash_type,
 				       default_style->line.dash_type);
-	gtk_table_attach (GTK_TABLE (table), w, 1, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_widget_set_halign (w, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), w, 1, 0, 2, 1);
 	g_signal_connect (G_OBJECT (w), "activate",
 			  G_CALLBACK (cb_line_dash_type_changed), state);
 
@@ -300,8 +304,8 @@ line_init (StylePrefState *state, gboolean enable, GOEditor *editor)
 		state->gui,
 		"line_color", "line_color_label",
 		G_CALLBACK (cb_line_color_changed));
-	gtk_table_attach (GTK_TABLE (table), w, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-	gtk_widget_show_all (table);
+	gtk_grid_attach (GTK_GRID (grid), w, 1, 1, 1, 1);
+	gtk_widget_show_all (grid);
 }
 
 static void cb_fill_background_color (GOSelector *selector, StylePrefState *state);
@@ -526,14 +530,14 @@ fill_image_init (StylePrefState *state)
 
 	sample = go_gtk_builder_get_widget (state->gui, "fill_image_sample");
 	gtk_widget_set_size_request (sample, HSCALE + 10, VSCALE + 10);
-	type   = go_gtk_builder_get_widget (state->gui, "fill_image_fit");
+	type   = go_gtk_builder_get_widget (state->gui, "fill-image-fit");
 
 	state->fill.image.image = NULL;
+	go_style_set_image_preview (style->fill.image.image, state);
 
 	if (GO_STYLE_FILL_IMAGE == style->fill.type) {
 		gtk_combo_box_set_active (GTK_COMBO_BOX (type),
 			style->fill.image.type);
-		go_style_set_image_preview (style->fill.image.image, state);
 		state->fill.image.image = style->fill.image.image;
 		if (state->fill.image.image)
 			g_object_ref (state->fill.image.image);
@@ -577,7 +581,6 @@ fill_update_visibilies (FillType type, StylePrefState *state)
 		      fill_infos[type].show_brightness, NULL);
 	g_object_set (state->fill.foreground_box, "visible",
 		      !fill_infos[type].show_brightness, NULL);
-	g_object_set (state->fill.extension_box, "visible", type != FILL_TYPE_NONE, NULL);
 
 	if (fill_infos[type].show_gradient) {
 		gtk_label_set_text (GTK_LABEL (state->fill.foreground_label), _("Start:"));
@@ -614,13 +617,11 @@ fill_init (StylePrefState *state, gboolean enable, GOEditor *editor)
 	FillType type;
 
 	if (!enable) {
-		gtk_widget_hide (go_gtk_builder_get_widget (state->gui, "fill_box"));
+		gtk_widget_hide (go_gtk_builder_get_widget (state->gui, "fill-grid"));
+		gtk_widget_hide (go_gtk_builder_get_widget (state->gui, "fill-label"));
 		return;
 	}
 
-	state->fill.extension_box = go_gtk_builder_get_widget (state->gui, "fill_extension_box");
-	go_editor_register_widget (editor, state->fill.extension_box);
-
 	state->fill.size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
 	fill_color_init (state);
@@ -630,7 +631,7 @@ fill_init (StylePrefState *state, gboolean enable, GOEditor *editor)
 		fill_image_init (state);
 	fill_update_selectors (state);
 
-	state->fill.notebook = go_gtk_builder_get_widget (state->gui, "fill_notebook");
+	state->fill.notebook = go_gtk_builder_get_widget (state->gui, "fill-notebook");
 
 	switch (state->style->fill.type) {
 		case GO_STYLE_FILL_PATTERN:
@@ -655,15 +656,16 @@ fill_init (StylePrefState *state, gboolean enable, GOEditor *editor)
 	}
 	fill_update_visibilies (type, state);
 
-	w = go_gtk_builder_get_widget (state->gui, "fill_type_menu");
+	w = go_gtk_builder_get_widget (state->gui, "fill-type-menu");
 	if (state->doc == NULL)
-		gtk_combo_box_remove_text (GTK_COMBO_BOX (w), FILL_TYPE_IMAGE);
+		gtk_combo_box_text_remove (GTK_COMBO_BOX_TEXT (w), FILL_TYPE_IMAGE);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (w), type);
 	g_signal_connect (G_OBJECT (w),
 		"changed",
 		G_CALLBACK (cb_fill_type_changed), state);
 
-	w = go_gtk_builder_get_widget (state->gui, "fill_box");
+	w = go_gtk_builder_get_widget (state->gui, "fill-grid");
+	go_editor_register_widget (editor, w);
 	gtk_widget_show (GTK_WIDGET (w));
 
 	g_object_unref (state->fill.size_group);
@@ -742,7 +744,7 @@ marker_init (StylePrefState *state, gboolean enable, GOEditor *editor, GOCmdCont
 	if (!enable)
 		return;
 
-	gui = go_gtk_builder_new ("go-style-prefs.ui", GETTEXT_PACKAGE, cc);
+	gui = go_gtk_builder_new_internal ("go-style-prefs.ui", GETTEXT_PACKAGE, cc);
 	if (gui == NULL)
 		return;
 
@@ -856,7 +858,7 @@ font_init (StylePrefState *state, guint32 enable, GOEditor *editor, GOCmdContext
 
 	g_return_if_fail (style->font.font != NULL);
 
-	gui = go_gtk_builder_new ("go-style-prefs.ui", GETTEXT_PACKAGE, cc);
+	gui = go_gtk_builder_new_internal ("go-style-prefs.ui", GETTEXT_PACKAGE, cc);
 	if (gui == NULL)
 		return;
 
@@ -904,7 +906,7 @@ text_layout_init (StylePrefState *state, guint32 enable, GOEditor *editor, GOCmd
 		style->text_layout.angle);
 	g_signal_connect (G_OBJECT (w), "rotation-changed",
 		G_CALLBACK (cb_angle_changed), state);
-	box = gtk_vbox_new (FALSE, 6);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_box_pack_start (GTK_BOX (box), w, TRUE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (box), 12);
 	go_editor_add_page (editor, box, _("Text"));
@@ -1032,13 +1034,13 @@ go_style_populate_editor (GOStyle *style,
 		state->doc = NULL;
 
 	if ((enable & (GO_STYLE_OUTLINE | GO_STYLE_LINE | GO_STYLE_FILL)) != 0) {
-		gui = go_gtk_builder_new ("go-style-prefs.ui", GETTEXT_PACKAGE, cc);
+		gui = go_gtk_builder_new_internal ("go-style-prefs.ui", GETTEXT_PACKAGE, cc);
 		if (gui == NULL) {
 			g_free (state);
 			return;
 		}
 		state->gui = gui;
-		w = go_gtk_builder_get_widget (gui, "go_style_prefs");
+		w = go_gtk_builder_get_widget (gui, "go-style-prefs");
 		g_object_set_data (G_OBJECT (w), "state", state);
 		g_signal_connect_swapped (G_OBJECT (w), "destroy", G_CALLBACK (go_style_pref_state_free), state);
 		go_editor_add_page (editor, w, _("Style"));
@@ -1272,20 +1274,6 @@ static struct {
 	{ GO_IMAGE_WALLPAPER,    "wallpaper" },
 };
 
-static gboolean
-bool_prop (xmlNode *node, char const *name, gboolean *res)
-{
-	char *str = xmlGetProp (node, name);
-	if (str != NULL) {
-		*res = g_ascii_tolower (*str) == 't' ||
-			g_ascii_tolower (*str) == 'y' ||
-			strtol (str, NULL, 0);
-		xmlFree (str);
-		return TRUE;
-	}
-	return FALSE;
-}
-
 static gboolean
 bool_sax_prop (char const *name, char const *id, char const *val, gboolean *res)
 {
@@ -1339,39 +1327,6 @@ image_tiling_as_str (GOImageType fstyle)
 	return "stretched";
 }
 
-static void
-go_style_line_load (xmlNode *node, GOStyleLine *line)
-{
-	char *str;
-	gboolean tmp;
-
-	str = xmlGetProp (node, "dash");
-	if (str != NULL) {
-		line->dash_type = go_line_dash_from_str (str);
-		xmlFree (str);
-	}
-	if (bool_prop (node, "auto-dash", &tmp))
-		line->auto_dash = tmp;
-	str = xmlGetProp (node, "width");
-	if (str != NULL) {
-		line->width = g_strtod (str, NULL);
-		/* For compatibility with older graphs, when dash_type
-		 * didn't exist */
-		if (line->width < 0.) {
-			line->width = 0.;
-			line->dash_type = GO_LINE_NONE;
-		}
-		xmlFree (str);
-	}
-	str = xmlGetProp (node, "color");
-	if (str != NULL) {
-		go_color_from_str (str, &line->color);
-		xmlFree (str);
-	}
-	if (bool_prop (node, "auto-color", &tmp))
-		line->auto_color = tmp;
-}
-
 static void
 go_style_line_sax_save (GsfXMLOut *output, char const *name,
 			 GOStyleLine const *line)
@@ -1445,6 +1400,7 @@ go_style_fill_sax_save (GsfXMLOut *output, GOStyle const *style)
 		gsf_xml_out_add_cstr_unchecked (output, "type",
 			image_tiling_as_str (style->fill.image.type));
 		gsf_xml_out_add_cstr (output, "name", go_image_get_name (style->fill.image.image));
+		gsf_xml_out_add_cstr (output, "type-name", G_OBJECT_TYPE_NAME (style->fill.image.image));
 		go_doc_save_image ((GODoc *) g_object_get_data (G_OBJECT (gsf_xml_out_get_output (output)), "document"), go_image_get_name (style->fill.image.image));
 		gsf_xml_out_end_element (output);
 		break;
@@ -1454,153 +1410,6 @@ go_style_fill_sax_save (GsfXMLOut *output, GOStyle const *style)
 	gsf_xml_out_end_element (output);
 }
 
-static void
-go_style_gradient_load (xmlNode *node, GOStyle *style)
-{
-	char    *str = xmlGetProp (node, "direction");
-	if (str != NULL) {
-		style->fill.gradient.dir
-			= go_gradient_dir_from_str (str);
-		xmlFree (str);
-	}
-	str = xmlGetProp (node, "start-color");
-	if (str != NULL) {
-		go_color_from_str (str, &style->fill.pattern.back);
-		xmlFree (str);
-	}
-	str = xmlGetProp (node, "brightness");
-	if (str != NULL) {
-		go_style_set_fill_brightness (style, g_strtod (str, NULL));
-		xmlFree (str);
-	} else {
-		str = xmlGetProp (node, "end-color");
-		if (str != NULL) {
-			go_color_from_str (str, &style->fill.pattern.fore);
-			xmlFree (str);
-		}
-	}
-}
-
-static void
-go_style_image_load (xmlNode *node, GOStyle *style)
-{
-	char *str = xmlGetProp (node, "type");
-	if (str != NULL) {
-		style->fill.image.type = str_as_image_tiling (str);
-		xmlFree (str);
-	}
-	/* TODO: load the pixels */
-}
-
-static void
-go_style_fill_load (xmlNode *node, GOStyle *style)
-{
-	xmlNode *ptr;
-	gboolean tmp;
-	char    *str = xmlGetProp (node, "type");
-
-	if (str == NULL)
-		return;
-	style->fill.type = str_as_fill_style (str);
-	xmlFree (str);
-
-	style->fill.auto_type = FALSE;
-
-	if (bool_prop (node, "auto-type", &tmp))
-		style->fill.auto_type = tmp;
-	if (bool_prop (node, "is-auto", &tmp))
-		style->fill.auto_back = tmp;
-	if (bool_prop (node, "auto-fore", &tmp))
-		style->fill.auto_fore = tmp;
-
-	switch (style->fill.type) {
-	case GO_STYLE_FILL_PATTERN:
-		for (ptr = node->xmlChildrenNode ;
-		     ptr != NULL ; ptr = ptr->next) {
-			if (xmlIsBlankNode (ptr) || ptr->name == NULL)
-				continue;
-			if (strcmp (ptr->name, "pattern") == 0) {
-				str = xmlGetProp (ptr, "type");
-				if (str != NULL) {
-					style->fill.pattern.pattern
-						= go_pattern_from_str (str);
-					xmlFree (str);
-				}
-				str = xmlGetProp (ptr, "fore");
-				if (str != NULL) {
-					go_color_from_str (str, &style->fill.pattern.fore);
-					xmlFree (str);
-				}
-				str = xmlGetProp (ptr, "back");
-				if (str != NULL) {
-					go_color_from_str (str, &style->fill.pattern.back);
-					xmlFree (str);
-				}
-			}
-		}
-		break;
-	case GO_STYLE_FILL_GRADIENT:
-		for (ptr = node->xmlChildrenNode ;
-		     ptr != NULL ; ptr = ptr->next) {
-			if (xmlIsBlankNode (ptr) || ptr->name == NULL)
-				continue;
-			if (strcmp (ptr->name, "gradient") == 0)
-				go_style_gradient_load (ptr, style);
-		}
-		break;
-	case GO_STYLE_FILL_IMAGE:
-		for (ptr = node->xmlChildrenNode ;
-		     ptr != NULL ; ptr = ptr->next) {
-			if (xmlIsBlankNode (ptr) || ptr->name == NULL)
-				continue;
-			if (strcmp (ptr->name, "image") == 0) {
-				go_style_image_load (ptr, style);
-			}
-		}
-		break;
-	default:
-		break;
-	}
-}
-
-static void
-go_style_marker_load (xmlNode *node, GOStyle *style)
-{
-	char *str;
-	GOColor c;
-	GOMarker *marker = go_marker_dup (style->marker.mark);
-
-	str = xmlGetProp (node, "shape");
-	if (str != NULL) {
-		style->marker.auto_shape = TRUE;
-		bool_prop (node, "auto-shape", &style->marker.auto_shape);
-		go_marker_set_shape (marker, go_marker_shape_from_str (str));
-		xmlFree (str);
-	}
-	str = xmlGetProp (node, "outline-color");
-	if (str != NULL) {
-		style->marker.auto_outline_color = TRUE;
-		bool_prop (node, "auto-outline", &style->marker.auto_outline_color);
-		if (go_color_from_str (str, &c))
-			go_marker_set_outline_color (marker, c);
-		xmlFree (str);
-	}
-	str = xmlGetProp (node, "fill-color");
-	if (str != NULL) {
-		style->marker.auto_fill_color = TRUE;
-		bool_prop (node, "auto-fill", &style->marker.auto_fill_color);
-		if (go_color_from_str (str, &c))
-			go_marker_set_fill_color (marker, c);
-		xmlFree (str);
-	}
-	str = xmlGetProp (node, "size");
-	if (str != NULL) {
-		go_marker_set_size (marker, g_strtod (str, NULL));
-		xmlFree (str);
-	}
-	go_style_set_marker (style, marker);
-}
-
 static void
 go_style_marker_sax_save (GsfXMLOut *output, GOStyle const *style)
 {
@@ -1620,30 +1429,6 @@ go_style_marker_sax_save (GsfXMLOut *output, GOStyle const *style)
 	gsf_xml_out_end_element (output);
 }
 
-static void
-go_style_font_load (xmlNode *node, GOStyle *style)
-{
-	char *str;
-	gboolean tmp;
-
-	str = xmlGetProp (node, "color");
-	if (str != NULL) {
-		go_color_from_str (str, &style->font.color);
-		xmlFree (str);
-	}
-	str = xmlGetProp (node, "font");
-	if (str != NULL) {
-		PangoFontDescription *desc;
-
-		desc = pango_font_description_from_string (str);
-		if (desc != NULL)
-			go_style_set_font_desc (style, desc);
-		xmlFree (str);
-	}
-	if (bool_prop (node, "auto-scale", &tmp))
-		style->font.auto_scale = tmp;
-}
-
 static void
 go_style_font_sax_save (GsfXMLOut *output, GOStyle const *style)
 {
@@ -1657,18 +1442,6 @@ go_style_font_sax_save (GsfXMLOut *output, GOStyle const *style)
 	gsf_xml_out_end_element (output);
 }
 
-static void
-go_style_text_layout_load (xmlNode *node, GOStyle *style)
-{
-	char *str;
-
-	str = xmlGetProp (node, "angle");
-	if (str != NULL) {
-		go_style_set_text_angle (style, g_strtod (str, NULL));
-		xmlFree (str);
-	}
-}
-
 static void
 go_style_text_layout_sax_save (GsfXMLOut *output, GOStyle const *style)
 {
@@ -1678,32 +1451,6 @@ go_style_text_layout_sax_save (GsfXMLOut *output, GOStyle const *style)
 	gsf_xml_out_end_element (output);
 }
 
-static gboolean
-go_style_persist_dom_load (GOPersist *gp, xmlNode *node)
-{
-	GOStyle *style = GO_STYLE (gp);
-	xmlNode *ptr;
-
-	/* while reloading no need to reapply settings */
-	for (ptr = node->xmlChildrenNode ; ptr != NULL ; ptr = ptr->next) {
-		if (xmlIsBlankNode (ptr) || ptr->name == NULL)
-			continue;
-		if (strcmp (ptr->name, "outline") == 0)
-			go_style_line_load (ptr, &style->line);
-		else if (strcmp (ptr->name, "line") == 0)
-			go_style_line_load (ptr, &style->line);
-		else if (strcmp (ptr->name, "fill") == 0)
-			go_style_fill_load (ptr, style);
-		else if (strcmp (ptr->name, "marker") == 0)
-			go_style_marker_load (ptr, style);
-		else if (strcmp (ptr->name, "font") == 0)
-			go_style_font_load (ptr, style);
-		else if (strcmp (ptr->name, "text_layout") == 0)
-			go_style_text_layout_load (ptr, style);
-	}
-	return TRUE;
-}
-
 static void
 go_style_sax_load_line (GsfXMLIn *xin, xmlChar const **attrs)
 {
@@ -1764,14 +1511,20 @@ go_style_sax_load_fill_image (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	GOStyle *style = GO_STYLE (xin->user_state);
 	GODoc *doc = (GODoc *) g_object_get_data (G_OBJECT (gsf_xml_in_get_input (xin)), "document");
+	xmlChar const *name = NULL, *type_name = NULL;
+	GType type;
 	g_return_if_fail (style->fill.type == GO_STYLE_FILL_NONE);
 	g_return_if_fail (GO_IS_DOC (doc));
-	/* TODO: load the pixels */
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "type")) {
+		if (0 == strcmp (attrs[0], "type"))
 			style->fill.image.type = str_as_image_tiling (attrs[1]);
-		} else if (0 == strcmp (attrs[0], "name"))
-			style->fill.image.image = g_object_ref (go_doc_image_fetch (doc, attrs[1]));
+		else if (0 == strcmp (attrs[0], "name"))
+			name = attrs[1];
+		else if (0 == strcmp (attrs[0], "type-name"))
+			type_name = attrs[1];
+	type = type_name? g_type_from_name (type_name): GO_TYPE_PIXBUF;
+	if (name && type)
+		style->fill.image.image = g_object_ref (go_doc_image_fetch (doc, name, type));
 	if (style->fill.image.image != NULL)
 		style->fill.type = GO_STYLE_FILL_IMAGE;
 }
@@ -1925,7 +1678,6 @@ go_style_persist_prep_sax (GOPersist *gp, GsfXMLIn *xin, xmlChar const **attrs)
 static void
 go_style_persist_init (GOPersistClass *iface)
 {
-	iface->dom_load = go_style_persist_dom_load;
 	iface->prep_sax = go_style_persist_prep_sax;
 	iface->sax_save = go_style_persist_sax_save;
 }
@@ -2005,6 +1757,16 @@ go_style_force_auto (GOStyle *style)
 	style->text_layout.auto_angle = TRUE;
 }
 
+gboolean
+go_style_is_auto (GOStyle *style)
+{
+	return style->marker.auto_shape && style->marker.auto_outline_color &&
+	       style->marker.auto_fill_color && style->line.auto_dash &&
+	       style->line.auto_color && style->fill.auto_type &&
+	       style->fill.auto_fore && style->fill.auto_back &&
+	       style->font.auto_scale && style->text_layout.auto_angle;
+}
+
 /**
  * go_style_set_marker :
  * @style : #GOStyle
@@ -2099,22 +1861,10 @@ go_style_set_text_angle (GOStyle *style, double angle)
 	style->text_layout.auto_angle = FALSE;
 }
 
-/**
- * go_style_create_cairo_pattern:
- * @style : #GOStyle
- * @cr: a cairo context
- *
- * Create a cairo_patern_t using the current style settings for filling.
- * A pattern will be created only if the style has the corresponding field
- * and if it is not set to a none constant.
- *
- * Returns: the pattern or NULL if it could not be created.
- **/
-cairo_pattern_t *
-go_style_create_cairo_pattern (GOStyle const *style, cairo_t *cr)
+void
+go_style_fill (GOStyle const *style, cairo_t *cr, gboolean preserve)
 {
-	cairo_pattern_t *cr_pattern;
-	cairo_matrix_t cr_matrix;
+	cairo_pattern_t *cr_pattern = NULL;
 	double x[3], y[3];
 	int w, h;
 
@@ -2137,19 +1887,19 @@ go_style_create_cairo_pattern (GOStyle const *style, cairo_t *cr)
 		{2, 2, 0, 1}
 	};
 
-	g_return_val_if_fail (GO_IS_STYLE (style), NULL);
-
-	if (style->fill.type == GO_STYLE_FILL_NONE)
-		return NULL;
-
 	cairo_fill_extents (cr, &x[0], &y[0], &x[1], &y[1]);
-	if (go_sub_epsilon (fabs (x[0] - x[1])) <=0.0 ||
-	    go_sub_epsilon (fabs (y[0] - y[1])) <=0.0)
-		return NULL;
+	if (!GO_IS_STYLE (style) ||
+	    go_sub_epsilon (fabs (x[0] - x[1])) <=0.0 ||
+	    go_sub_epsilon (fabs (y[0] - y[1])) <=0.0) {
+		if (!preserve)
+			cairo_new_path (cr);
+		return;
+	    }
 
 	switch (style->fill.type) {
 		case GO_STYLE_FILL_PATTERN:
-			return go_pattern_create_cairo_pattern (&style->fill.pattern, cr);
+			cr_pattern = go_pattern_create_cairo_pattern (&style->fill.pattern, cr);
+			break;
 
 		case GO_STYLE_FILL_GRADIENT:
 			x[2] = (x[1] - x[0]) / 2.0 + x[0];
@@ -2164,48 +1914,71 @@ go_style_create_cairo_pattern (GOStyle const *style, cairo_t *cr)
 				GO_COLOR_TO_CAIRO (style->fill.pattern.back));
 			cairo_pattern_add_color_stop_rgba (cr_pattern, 1,
 				GO_COLOR_TO_CAIRO (style->fill.pattern.fore));
-			return cr_pattern;
+			break;
 
 		case GO_STYLE_FILL_IMAGE:
-			if (style->fill.image.image == NULL)
-				return cairo_pattern_create_rgba (1, 1, 1, 1);
-
-			cr_pattern = go_image_create_cairo_pattern (style->fill.image.image);
-			if (cr_pattern == NULL) {
-				/* don't reference anymore an invalid image */
-				((GOStyle *) style)->fill.image.image = NULL;
-				return cairo_pattern_create_rgba (1, 1, 1, 1);
+			if (!GO_IS_IMAGE (style->fill.image.image))
+				cr_pattern = cairo_pattern_create_rgba (1, 1, 1, 1);
+			else {
+				cairo_save (cr);
+				if (preserve)
+					cairo_clip_preserve (cr);
+				else
+					cairo_clip (cr);
+				g_object_get (style->fill.image.image, "width", &w, "height", &h, NULL);
+				switch (style->fill.image.type) {
+					case GO_IMAGE_CENTERED:
+						cairo_translate (cr,
+						                 (x[1] - x[0] - w) / 2 + x[0],
+						                 (y[1] - y[0] - h) / 2 + y[0]);
+						go_image_draw (style->fill.image.image, cr);
+						break;
+					case GO_IMAGE_STRETCHED:
+						cairo_translate (cr, x[0], y[0]);
+						cairo_scale (cr, (x[1] - x[0]) / w, (y[1] - y[0]) / h);
+						go_image_draw (style->fill.image.image, cr);
+						break;
+					case GO_IMAGE_CENTERED_WALLPAPER: {
+						int n = go_fake_floor ((x[1] - x[0]) / w);
+						x[0] -= w - (x[1] - x[0] - n * w) / 2.;
+						n = go_fake_floor ((y[1] - y[0]) / h);
+						y[0] -= h - (y[1] - y[0] - n * h) / 2.;
+					}
+					case GO_IMAGE_WALLPAPER: {
+						double cx = x[0], cy;
+						while (cx < x[1]) {
+							cy = y[0];
+							while (cy < y[1]) {
+								cairo_save (cr);
+								cairo_translate (cr, cx, cy);
+								go_image_draw (style->fill.image.image, cr);
+								cairo_restore (cr);
+								cy += h;
+							}
+							cx += w;
+						}
+						break;
+					}
+				}
+				cairo_restore (cr);
+				return;
 			}
-			g_object_get (style->fill.image.image, "width", &w, "height", &h, NULL);
-			switch (style->fill.image.type) {
-				case GO_IMAGE_CENTERED:
-					cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_NONE);
-					cairo_matrix_init_translate (&cr_matrix,
-								     -(x[1] - x[0] - w) / 2 - x[0],
-								     -(y[1] - y[0] - h) / 2 - y[0]);
-					cairo_pattern_set_matrix (cr_pattern, &cr_matrix);
-					break;
-				case GO_IMAGE_STRETCHED:
-					cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_NONE);
-					cairo_matrix_init_scale (&cr_matrix,
-								 w / (x[1] - x[0]),
-								 h / (y[1] - y[0]));
-					cairo_matrix_translate (&cr_matrix, -x[0], -y[0]);
-					cairo_pattern_set_matrix (cr_pattern, &cr_matrix);
-					break;
-				case GO_IMAGE_WALLPAPER:
-					cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_REPEAT);
-					cairo_matrix_init_translate (&cr_matrix, -x[0], -y[0]);
-					cairo_pattern_set_matrix (cr_pattern, &cr_matrix);
-					break;
-			}
-			return cr_pattern;
 
 		case GO_STYLE_FILL_NONE:
-			return NULL;
+			if (!preserve)
+				cairo_new_path (cr);
+			break;
 	}
 
-	return NULL;
+	if (cr_pattern) {
+		cairo_set_source (cr, cr_pattern);
+		cairo_pattern_destroy (cr_pattern);
+
+		if (preserve)
+			cairo_fill_preserve (cr);
+		else
+			cairo_fill (cr);
+	}
 }
 
 gboolean
