@@ -30,7 +30,6 @@
 #include <goffice/utils/go-persist.h>
 #include <goffice/utils/go-styled-object.h>
 #include <goffice/app/go-plugin.h>
-#include <goffice/gtk/go-gtk-compat.h>
 
 #include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
@@ -307,6 +306,7 @@ gog_minmax_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogMinMaxPlot const *model = GOG_MINMAX_PLOT (view->model);
 	GogPlot1_5d const *gog_1_5d_model = GOG_PLOT1_5D (view->model);
 	GogSeries1_5d const *series;
+	GogSeries const *base_series;
 	GogAxisMap *x_map, *y_map;
 	gboolean is_vertical = ! (model->horizontal);
 	double *max_vals, *min_vals;
@@ -346,14 +346,15 @@ gog_minmax_view_render (GogView *view, GogViewAllocation const *bbox)
 
 	for (ptr = gog_1_5d_model->base.series ; ptr != NULL ; ptr = ptr->next) {
 		series = ptr->data;
-		if (!gog_series_is_valid (GOG_SERIES (series)))
+		base_series = GOG_SERIES (series);
+		if (!gog_series_is_valid (base_series))
 			continue;
 		style = go_styled_object_get_style (GO_STYLED_OBJECT (series));
 		x = offset;
-		min_vals = go_data_get_values (series->base.values[1].data);
-		n = go_data_get_vector_size (series->base.values[1].data);
-		max_vals = go_data_get_values (series->base.values[2].data);
-		tmp = go_data_get_vector_size (series->base.values[2].data);
+		min_vals = go_data_get_values (base_series->values[1].data);
+		n = go_data_get_vector_size (base_series->values[1].data);
+		max_vals = go_data_get_values (base_series->values[2].data);
+		tmp = go_data_get_vector_size (base_series->values[2].data);
 		if (n > tmp)
 			n = tmp;
 		mpath = go_path_new ();
